#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022/9/21 09:01:40

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import os
import numpy as np
from tqdm import tqdm
from torch.utils.data import Dataset
import laok.util.log as klog
# ===============================================================================
r'''
split = 
    train
    test

'''
# ===============================================================================

class S3DISDataset(Dataset):
    def __init__(self, root, split='train', num_point=4096, test_area=5, block_size=1.0, sample_rate=1.0, transform=None):

        klog.debug(f'===== {self.__class__.__name__}')
        klog.debug(f"root= {root}")
        klog.debug(f"split= {split} num_point= {num_point} test_area={test_area} block_size={block_size} sample_rate={sample_rate} transform={transform}")

        self.num_point = num_point
        self.block_size = block_size
        self.transform = transform
        rooms = sorted(os.listdir(root))
        rooms = [room for room in rooms if 'Area_' in room]

        klog.debug(f'rooms= {rooms}')

        if split == 'train':
            rooms_split = [room for room in rooms if 'Area_{}'.format(test_area) not in room]
        else:
            rooms_split = [room for room in rooms if 'Area_{}'.format(test_area) in room]

        klog.debug(f'rooms_split= {rooms_split}')

        self.room_points, self.room_labels = [], []
        self.room_coord_min, self.room_coord_max = [], []
        num_point_all = []
        labelweights = np.zeros(13)

        for room_name in tqdm(rooms_split, total=len(rooms_split)):
            room_path = os.path.join(root, room_name)
            room_data = np.load(room_path)  # xyzrgbl, N*7
            points, labels = room_data[:, 0:6], room_data[:, 6]  # xyzrgb, N*6; l, N
            tmp, _ = np.histogram(labels, range(14))
            labelweights += tmp
            coord_min, coord_max = np.amin(points, axis=0)[:3], np.amax(points, axis=0)[:3]
            self.room_points.append(points), self.room_labels.append(labels)
            self.room_coord_min.append(coord_min), self.room_coord_max.append(coord_max)
            num_point_all.append(labels.size)
        labelweights = labelweights.astype(np.float32)
        labelweights = labelweights / np.sum(labelweights)
        self.labelweights = np.power(np.amax(labelweights) / labelweights, 1 / 3.0)
        print(self.labelweights)
        sample_prob = num_point_all / np.sum(num_point_all)
        num_iter = int(np.sum(num_point_all) * sample_rate / num_point)
        room_idxs = []
        for index in range(len(rooms_split)):
            room_idxs.extend([index] * int(round(sample_prob[index] * num_iter)))
        self.room_idxs = np.array(room_idxs)
        print("Totally {} samples in {} set.".format(len(self.room_idxs), split))

    def __getitem__(self, idx):
        room_idx = self.room_idxs[idx]
        points = self.room_points[room_idx]   # N * 6
        labels = self.room_labels[room_idx]   # N
        N_points = points.shape[0]

        while (True):
            center = points[np.random.choice(N_points)][:3]
            block_min = center - [self.block_size / 2.0, self.block_size / 2.0, 0]
            block_max = center + [self.block_size / 2.0, self.block_size / 2.0, 0]
            point_idxs = np.where((points[:, 0] >= block_min[0]) & (points[:, 0] <= block_max[0]) & (points[:, 1] >= block_min[1]) & (points[:, 1] <= block_max[1]))[0]
            if point_idxs.size > 1024:
                break

        if point_idxs.size >= self.num_point:
            selected_point_idxs = np.random.choice(point_idxs, self.num_point, replace=False)
        else:
            selected_point_idxs = np.random.choice(point_idxs, self.num_point, replace=True)

        # normalize
        selected_points = points[selected_point_idxs, :]  # num_point * 6
        current_points = np.zeros((self.num_point, 9))  # num_point * 9
        current_points[:, 6] = selected_points[:, 0] / self.room_coord_max[room_idx][0]
        current_points[:, 7] = selected_points[:, 1] / self.room_coord_max[room_idx][1]
        current_points[:, 8] = selected_points[:, 2] / self.room_coord_max[room_idx][2]
        selected_points[:, 0] = selected_points[:, 0] - center[0]
        selected_points[:, 1] = selected_points[:, 1] - center[1]
        selected_points[:, 3:6] /= 255.0
        current_points[:, 0:6] = selected_points
        current_labels = labels[selected_point_idxs]
        if self.transform is not None:
            current_points, current_labels = self.transform(current_points, current_labels)
        return current_points, current_labels

    def __len__(self):
        return len(self.room_idxs)

class ScannetDatasetWholeScene():
    # prepare to give prediction on each points
    def __init__(self, root, block_points=4096, split='test', test_area=5, stride=0.5, block_size=1.0, padding=0.001):
        self.block_points = block_points
        self.block_size = block_size
        self.padding = padding
        self.root = root
        self.split = split
        self.stride = stride
        self.scene_points_num = []
        assert split in ['train', 'test']
        if self.split == 'train':
            self.file_list = [d for d in os.listdir(root) if d.find('Area_%d' % test_area) is -1]
        else:
            self.file_list = [d for d in os.listdir(root) if d.find('Area_%d' % test_area) is not -1]
        self.scene_points_list = []
        self.semantic_labels_list = []
        self.room_coord_min, self.room_coord_max = [], []
        for file in self.file_list:
            data = np.load(root + file)
            points = data[:, :3]
            self.scene_points_list.append(data[:, :6])
            self.semantic_labels_list.append(data[:, 6])
            coord_min, coord_max = np.amin(points, axis=0)[:3], np.amax(points, axis=0)[:3]
            self.room_coord_min.append(coord_min), self.room_coord_max.append(coord_max)
        assert len(self.scene_points_list) == len(self.semantic_labels_list)

        labelweights = np.zeros(13)
        for seg in self.semantic_labels_list:
            tmp, _ = np.histogram(seg, range(14))
            self.scene_points_num.append(seg.shape[0])
            labelweights += tmp
        labelweights = labelweights.astype(np.float32)
        labelweights = labelweights / np.sum(labelweights)
        self.labelweights = np.power(np.amax(labelweights) / labelweights, 1 / 3.0)

    def __getitem__(self, index):
        point_set_ini = self.scene_points_list[index]
        points = point_set_ini[:,:6]
        labels = self.semantic_labels_list[index]
        coord_min, coord_max = np.amin(points, axis=0)[:3], np.amax(points, axis=0)[:3]
        grid_x = int(np.ceil(float(coord_max[0] - coord_min[0] - self.block_size) / self.stride) + 1)
        grid_y = int(np.ceil(float(coord_max[1] - coord_min[1] - self.block_size) / self.stride) + 1)
        data_room, label_room, sample_weight, index_room = np.array([]), np.array([]), np.array([]),  np.array([])
        for index_y in range(0, grid_y):
            for index_x in range(0, grid_x):
                s_x = coord_min[0] + index_x * self.stride
                e_x = min(s_x + self.block_size, coord_max[0])
                s_x = e_x - self.block_size
                s_y = coord_min[1] + index_y * self.stride
                e_y = min(s_y + self.block_size, coord_max[1])
                s_y = e_y - self.block_size
                point_idxs = np.where(
                    (points[:, 0] >= s_x - self.padding) & (points[:, 0] <= e_x + self.padding) & (points[:, 1] >= s_y - self.padding) & (
                                points[:, 1] <= e_y + self.padding))[0]
                if point_idxs.size == 0:
                    continue
                num_batch = int(np.ceil(point_idxs.size / self.block_points))
                point_size = int(num_batch * self.block_points)
                replace = False if (point_size - point_idxs.size <= point_idxs.size) else True
                point_idxs_repeat = np.random.choice(point_idxs, point_size - point_idxs.size, replace=replace)
                point_idxs = np.concatenate((point_idxs, point_idxs_repeat))
                np.random.shuffle(point_idxs)
                data_batch = points[point_idxs, :]
                normlized_xyz = np.zeros((point_size, 3))
                normlized_xyz[:, 0] = data_batch[:, 0] / coord_max[0]
                normlized_xyz[:, 1] = data_batch[:, 1] / coord_max[1]
                normlized_xyz[:, 2] = data_batch[:, 2] / coord_max[2]
                data_batch[:, 0] = data_batch[:, 0] - (s_x + self.block_size / 2.0)
                data_batch[:, 1] = data_batch[:, 1] - (s_y + self.block_size / 2.0)
                data_batch[:, 3:6] /= 255.0
                data_batch = np.concatenate((data_batch, normlized_xyz), axis=1)
                label_batch = labels[point_idxs].astype(int)
                batch_weight = self.labelweights[label_batch]

                data_room = np.vstack([data_room, data_batch]) if data_room.size else data_batch
                label_room = np.hstack([label_room, label_batch]) if label_room.size else label_batch
                sample_weight = np.hstack([sample_weight, batch_weight]) if label_room.size else batch_weight
                index_room = np.hstack([index_room, point_idxs]) if index_room.size else point_idxs
        data_room = data_room.reshape((-1, self.block_points, data_room.shape[1]))
        label_room = label_room.reshape((-1, self.block_points))
        sample_weight = sample_weight.reshape((-1, self.block_points))
        index_room = index_room.reshape((-1, self.block_points))
        return data_room, label_room, sample_weight, index_room

    def __len__(self):
        return len(self.scene_points_list)

if __name__ == '__main__':
    import laok.util.conf as kconf
    root = kconf.get_conf("datasets").StanfordIndoor3d
    num_point, test_area, block_size, sample_rate = 4096, 5, 1.0, 0.01

    point_data = S3DISDataset(root=root, split='train', num_point=num_point, test_area=test_area, block_size=block_size, sample_rate=sample_rate, transform=None)
    print('point data size:', point_data.__len__())
    print('point data 0 shape:', point_data.__getitem__(0)[0].shape)
    print('point label 0 shape:', point_data.__getitem__(0)[1].shape)
    import torch, time, random
    manual_seed = 123
    random.seed(manual_seed)
    np.random.seed(manual_seed)
    torch.manual_seed(manual_seed)
    torch.cuda.manual_seed_all(manual_seed)
    def worker_init_fn(worker_id):
        random.seed(manual_seed + worker_id)
    train_loader = torch.utils.data.DataLoader(point_data, batch_size=16, shuffle=True, num_workers=16, pin_memory=True, worker_init_fn=worker_init_fn)
    for idx in range(4):
        end = time.time()
        for i, (input, target) in enumerate(train_loader):
            print('time: {}/{}--{}'.format(i+1, len(train_loader), time.time() - end))
            end = time.time()




#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2025/5/16 01:05:56

@author: Liu Kuan
@copyright: Apache License, Version 2.0
'''
import numpy as np
# ===============================================================================
r'''
'''
# ===============================================================================

def euclidean(x, y):
    # 欧氏距离
    el = np.sqrt(np.sum((x - y) ** 2))
    return el


def manhattan(x, y):
    # 曼哈顿距离
    mh = np.sum(np.abs(x - y))
    return mh


def chebyshev(x, y):
    # 切比雪夫距离
    cs = np.max(np.abs(x - y))
    return cs


def minkowski(x, y, p):
    # 闵可夫斯基距离
    ms = np.sum(np.abs(x - y) ** p) ** (1 / p)
    return ms


def hamming(x, y):
    # 汉明距离
    hm = np.sum(x != y) / len(x)
    return hm


if __name__ == '__main__':
    a = np.array([1, 2, 3])
    b = np.array([4, 5, 6])
    print('a,b之间的欧氏距离为：', euclidean(a, b))




#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/8/30 10:55:47

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import os
#===============================================================================
r'''
'''
#===============================================================================
__all__ = [
           'cur_line_num', 'cur_func_name', 'cur_func_kws',
           ]

def cur_line_num(depth = 1):
    '''获取当前行号'''
    return sys._getframe(depth).f_lineno

def cur_func_name(depth = 1):
    '''获取当前函数名字 '''
    return sys._getframe(depth).f_code.co_name

def cur_func_kws(depth = 1, skip_names = ("self", "cls")):
    ''' 获取函数的参数 '''
    _locals = sys._getframe(depth).f_locals.copy()
    if skip_names:
        if isinstance(skip_names, str):
            _locals.pop(skip_names, None)
        else:
            for name in skip_names:
                _locals.pop(name, None)
    return _locals



#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022/9/17 23:37:17

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import numpy as np
from torch.utils.data import Dataset
import laok.util.log as klog
from laok.cv3d.transform.sample import sample_cld_random, sample_cld_index
import laok.util.fs as kfs
# ===============================================================================
r'''
'''
# ===============================================================================

class UstcLidar(Dataset):

    def __init__(self, root, train=True, num_point=3000, transform=None, show_info=True):
        self.root = root
        self.num_point = num_point
        self.show_info = show_info
        self.transform = transform
        self.show_info = show_info
        self.train = train

        if show_info:
            klog.debug(f'===== {self.__class__.__name__}')
            klog.debug(f'train={train}, root={root}, transform={transform}')
            klog.debug(f'num_point={num_point}, show_info={show_info}')

        cat = []
        datapath = {}
        idx = 0
        for i, dirname in enumerate(kfs.dirs_cur(root)):
            name = kfs.path_stem(dirname).split('_')[1]
            cat.append(name)
            for fname in kfs.files_under(dirname, '.bin'):
                datapath[idx] = (i, fname)
                idx += 1

        self.datapath = datapath
        self.cat = cat
        self.classes = dict(zip(self.cat, range(len(self.cat))))

        if self.show_info:
            klog.debug(f'the size of data is {len(self.datapath)}, num_classes={len(self.classes)}')
            klog.debug(f'classes={self.classes}')

    def __len__(self):
        return len(self.datapath)

    def _get_item(self, index):
        cls, fname = self.datapath[index]
        label = np.array([cls]).astype(np.int32)

        # if self.show_info:
        #     klog.info(f'load {fname}')

        point_set = np.fromfile(fname, dtype=np.float32).reshape(-1, 4)

        if self.train:
            point_set = sample_cld_random(point_set, self.num_point)
        else:
            point_set = sample_cld_index(point_set, self.num_point)

        if self.transform:
            point_set = self.transform(point_set)

        return point_set, label[0]

    def __getitem__(self, index):
        return self._get_item(index)

# if __name__ == '__main__':
#     import torch
#     import laok.util.torch_ as ktorch
#     import laok.util.conf as kconf
#
#     dataset = UstcLidar(kconf.get_conf('datasets').Ustc,  num_point=3000, show_info=False)
#     dataloader = torch.utils.data.DataLoader(dataset, batch_size=2, shuffle=False)
#     for point, label in dataloader:
#         ktorch.pr(point)
#         ktorch.pr(label)
#
#         kcv3.show_cld_xyz(point[0])
#         break



#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/4/19 22:39:34
@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import os
from ..fs import path_replace_basename
from ..alg.addict import Dict
#===============================================================================
r'''全局配置
'''
#===============================================================================
__all__ = ['g_conf', 'conf_update', 'conf_merge', 'conf_merge_file']
g_conf = Dict()

def conf_update(**kws):
    g_conf.update(**kws)

def conf_merge(data):
    g_conf.update(data)

def conf_merge_file(fname):
    if fname.endswith('.json'):
        import json
        with open(fname, encoding='utf8') as f:
            _data = json.load(f)
            conf_merge(_data)
    elif fname.endswith('.yaml'):
        import yaml
        with open(fname, encoding='utf8') as f:
            _data = yaml.safe_load(f)
            conf_merge(_data)

#默认配置文件
_conf_file = path_replace_basename(__file__, 'conf.yaml')
conf_merge_file(_conf_file)

#读取环境变量配置
_env_path = os.environ.get('LAOK_CONF_PATH')
if _env_path and os.path.exists(_env_path):
    conf_merge_file(_env_path)




#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/7/11 19:26:54

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import cv2
import numpy as np
# ===============================================================================
r'''
'''
# ===============================================================================
__all__ = ['show_image', 'wait_key']

def show_image(mat, winname='img',
               winflag = cv2.WINDOW_NORMAL,
               title = ""):

    #窗口名字
    cv2.namedWindow(winname, winflag)

    # 标题
    if title:
        cv2.setWindowTitle(winname, title)

    if mat.dtype == bool: #转换bool 类型
        mat = mat.astype(np.uint8) * 255
    elif mat.dtype != np.uint8:  # 转换 其它类型
        mat = cv2.normalize(mat, None, alpha=0, beta=255,
                            norm_type=cv2.NORM_MINMAX, dtype=cv2.CV_8U)

    cv2.imshow(winname, mat)

def wait_key(wait_delay=0):
    '''支持 Esc 退出
    '''
    key = cv2.waitKey(wait_delay)
    if key & 0xFF == 27:  # 'Esc' to quit
        exit()
    return key


#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/8/2 23:13:49

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import numpy as np
# ===============================================================================
r'''
'''
# ===============================================================================
__all__ = ['print_data']

def print_data(data, **kws):
    '''print detail data information
    '''
    if isinstance(data, np.ndarray):
        _print_arrary(data, **kws)
    else:
        msg = f'type[{type(data)}] id[{id(data)}]\n ' \
              f'{data}'
        print(msg)

def _print_arrary(data, show_content=True):
    msg = f'type[{type(data)}] ' \
          f'shape[{data.shape}]  ' \
          f'dtype[{data.dtype}]  ' \
          f'id[{id(data)}]\n'
    if show_content:
        msg += f'{data}'
    print(msg)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/8/2 22:28:48

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
from sklearn import cluster
from sklearn import mixture
# ===============================================================================
r'''
'''
# ===============================================================================
__all__ = [
 'ClusterAffinityPropagation',
 'ClusterAgglomerative',
 'ClusterBayesianGaussianMixture',
 'ClusterBirch',
 'ClusterBisectingKMeans',
 'ClusterDBSCAN',
 'ClusterGaussianMixture',
 'ClusterKMeans',
 'ClusterMeanShift',
 'ClusterMiniBatchKMeans',
 'ClusterOPTICS',
 'ClusterSpectral',
 'ClusterSpectralBi',
 'ClusterSpectralCo']

# cluster
ClusterAffinityPropagation = cluster.AffinityPropagation
ClusterAgglomerative = cluster.AgglomerativeClustering
ClusterBirch = cluster.Birch
ClusterDBSCAN = cluster.DBSCAN
ClusterKMeans = cluster.KMeans
ClusterBisectingKMeans = cluster.BisectingKMeans
ClusterMiniBatchKMeans = cluster.MiniBatchKMeans
ClusterMeanShift = cluster.MeanShift
ClusterOPTICS = cluster.OPTICS
ClusterSpectral = cluster.SpectralClustering
ClusterSpectralCo = cluster.SpectralCoclustering
ClusterSpectralBi = cluster.SpectralBiclustering

# mixture
ClusterGaussianMixture = mixture.GaussianMixture
ClusterBayesianGaussianMixture = mixture.BayesianGaussianMixture



#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022/9/17 23:37:17

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import os
import numpy as np
from torch.utils.data import Dataset
from laok import log_info
# ===============================================================================
r'''
'''
# ===============================================================================

class Modelnet(Dataset):
    def __init__(self, root, split='train', num_point=1024, num_category=40, transforms=None, cached=False, show_info=False):
        self.root = root
        self.npoints = num_point
        self.num_category = num_category
        self.transforms = transforms
        self.split = split

        log_info(f'===== {self.__class__.__name__}')
        log_info(f'root={root}')
        log_info(f'split={split}, num_point={num_point}, num_category={num_category}')

        if self.num_category == 10:
            self.catfile = os.path.join(self.root, 'modelnet10_shape_names.txt')
        else:
            self.catfile = os.path.join(self.root, 'modelnet40_shape_names.txt')

        log_info(f'catfile={self.catfile}')

        self.cat = [line.rstrip() for line in open(self.catfile)]
        self.classes = dict(zip(self.cat, range(len(self.cat))))

        log_info(f'cat={self.cat}')
        log_info(f'classes={self.classes}')

        shape_ids = {}
        if self.num_category == 10:
            shape_ids['train'] = [line.rstrip() for line in open(os.path.join(self.root, 'modelnet10_train.txt'))]
            shape_ids['test'] = [line.rstrip() for line in open(os.path.join(self.root, 'modelnet10_test.txt'))]
        else:
            shape_ids['train'] = [line.rstrip() for line in open(os.path.join(self.root, 'modelnet40_train.txt'))]
            shape_ids['test'] = [line.rstrip() for line in open(os.path.join(self.root, 'modelnet40_test.txt'))]

        assert (split == 'train' or split == 'test')
        shape_names = ['_'.join(x.split('_')[0:-1]) for x in shape_ids[split]]
        self.datapath = [(shape_names[i], os.path.join(self.root, shape_names[i], shape_ids[split][i]) + '.txt') for i
                         in range(len(shape_ids[split]))]
        log_info(f'The size of {split} data is {len(self.datapath)}')
        self.show_info = show_info

    def __len__(self):
        return len(self.datapath)

    def _get_item(self, index):
        name, fname = self.datapath[index]
        cls = self.classes[name]
        label = np.array([cls]).astype(np.int32)
        if self.show_info:
            log_info(f'load {fname}')
        point_set = np.loadtxt(fname, delimiter=',').astype(np.float32)
        # point_set = kcv3.farthest_point_sample(point_set, self.npoints)
        if self.split == 'train':
            point_set = kcv3.random_sample(point_set, self.npoints)
        else:
            point_set = point_set[:self.npoints, :]

        point_set[:, 0:3] = kcv3.pc_normalize(point_set[:, 0:3])
        if self.transforms:
            point_set = self.transforms(point_set)
        return point_set, label[0]

    def __getitem__(self, index):
        return self._get_item(index)

if __name__ == '__main__':
    import torch
    import laok.util.torch_ as ktorch
    import laok.util.conf as kconf

    dataset = Modelnet(kconf.get_conf('datasets').Modelnet ,  num_point=1024, show_info=True, cached=True)
    # DataLoader = torch.utils.data.DataLoader(dataset, batch_size=2, shuffle=False)
    # for point, label in dataset:
    #     ktorch.pr(point)
    #     ktorch.pr(label)

    #     kcv3.show_cld_xyz(point[0])
    #     break




def show_cld_xyz(*arr, color=(255,255,255)):
    from .vtk_impl import _show_cld
    _show_cld(*arr, data_fmt="xyz", color=color)

def show_cld_xyznormal(*arr, color=(255,255,255)):
    from .vtk_impl import _show_cld
    _show_cld(*arr, data_fmt="xyzn", color=color)

def show_cld_xyzrgb(arr):
    from .vtk_impl import _show_cld
    _show_cld(*arr, data_fmt="xyzrgb")


#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/8/28 19:43:53

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import laok
import laok.ext.cv2_ as cv2_
import imagesize
#===============================================================================
r'''
'''
#===============================================================================
__all__ = ['resize_to_width_under',
            'resize_to_height_under',
            'resize_abs_under',
           'resize_rel_under',
           ]

def resize_to_width_under(img_dir, w, interpolation=1):
    for img_file in laok.image_under(img_dir):
        w2, h2 = imagesize.get(img_file)
        if w2 == w:
            continue
        img = cv2_.read_image(img_file)
        img = cv2_.resize_to_width(img, w, interpolation)
        cv2_.write_image(img_file, img)
        print(f'resize file:{img_file}')

def resize_to_height_under(img_dir, h, interpolation=1):
    for img_file in laok.image_under(img_dir):
        w2, h2 = imagesize.get(img_file)
        if h2 == h:
            continue
        img = cv2_.read_image(img_file)
        img = cv2_.resize_to_height(img, h, interpolation)
        cv2_.write_image(img_file, img)
        print(f'resize file:{img_file}')
def resize_abs_under(img_dir, w, h, interpolation=1):
    for img_file in laok.image_under(img_dir):
        w2, h2 = imagesize.get(img_file)
        if w2 == w and h2 == h:
            continue
        img = cv2_.read_image(img_file)
        img = cv2_.resize_abs(img, w, h, interpolation)
        cv2_.write_image(img_file, img)
        print(f'resize file:{img_file}')
def resize_rel_under(img_dir, rw, rh, interpolation=1):
    for img_file in laok.image_under(img_dir):
        img = cv2_.read_image(img_file)
        img = cv2_.resize_rel(img, rw, rh, interpolation)
        cv2_.write_image(img_file, img)
        print(f'resize file:{img_file}')


#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022/6/1 15:25:00

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
from io import StringIO
from pprint import pprint
#===============================================================================
'''     
'''
#===============================================================================
__all__ = [ 'pprint_str', 'val_to_list',
            'dict_to_str', 'parse_index_list']

def pprint_str(obj, indent=4, width=80, depth=None, compact=False, sort_dicts=True):
    s = StringIO()
    pprint(obj, s, indent=indent, width=width, depth=depth, compact=compact, sort_dicts=sort_dicts)
    return s.getvalue()

def val_to_list(val, sep=';'):
    if val is None:
        return ()
    if isinstance(val, (list,tuple)):
        return val
    if isinstance(val, str):
        return val.split(sep)
    return val



def dict_to_str(obj, start=10, stop=10):
    sz = len(obj)
    _istart = start
    _iend = sz - stop
    _once = True

    ss = StringIO()
    for i, (k, v) in enumerate(obj.items()):
        if i < _istart or i >= _iend:
            ss.write(f'{k}={v}\n')
        else:
            if _once:
                ss.write('... ...\n')
                _once = False
    return ss.getvalue()

def parse_index_list(val_list):
    idx_list = []
    for v in val_list:
        if isinstance(v, int):
            idx_list.append(v)
        elif isinstance(v, str):
            segs = v.split('-')
            if len(segs) == 1:
                idx_list.append(int(v))
            elif len(segs) == 2:
                _v1, _v2 = int(segs[0]), int(segs[1])
                idx_list.extend(range(_v1, _v2+1))
    return idx_list




#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022/10/17 20:23:23

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import torch
# ===============================================================================
r'''
'''
# ===============================================================================
__all__ = ['save_jit_model', 'load_jit_model']

def save_jit_model(model, input, filename):
    if isinstance(model, torch.jit.TracedModule):
        traced_model = model
    else:
        try:
            mode = model.training
            model.eval()
            traced_model = torch.jit.trace(model, input)
        finally:
            model.train(mode)

    traced_model.save(filename)
    return traced_model

def load_jit_model(filename):
    ''' load trace model
    '''
    return torch.jit.load(filename)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2025/10/11 12:13:52

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import matplotlib.pyplot as plt
# ===============================================================================
r'''
'''
# ===============================================================================

def show_tsne(features, labels, **tsne_kws):
    from sklearn.manifold import TSNE  # 从sklearn导入t-SNE算法

    # 使用t-SNE降维到2维
    n_components = tsne_kws.pop('n_components', 2)  # 降维到2维
    perplexity = tsne_kws.pop('perplexity', 30)  # 困惑度为30(控制局部和全局结构的平衡)通常建议值在5-50之间
    max_iter = tsne_kws.pop('max_iter', 300)  # 最大迭代300次
    random_state = tsne_kws.pop('random_state', 42) # 随机种子42
    tsne = TSNE(n_components=n_components,
                perplexity=perplexity,
                max_iter=max_iter,
                random_state=random_state, **tsne_kws)

    features_low = tsne.fit_transform(features)

    # 绘制二维t-SNE图
    # 创建散点图：x,y轴为降维后的两个维度, 颜色根据标签区分,使用viridis颜色映射, 透明度0.7, 点大小30
    scatter = plt.scatter(features_low[:, 0], features_low[:, 1], c=labels,
                          cmap='viridis', alpha=0.7, s=30)
    # 添加图例和标题
    plt.legend(*scatter.legend_elements(), title="Classes")  # 添加图例
    plt.title('t-SNE Visualization')  # 标题
    plt.xlabel('t-SNE dim 1')  # x轴标签
    plt.ylabel('t-SNE dim 2')  # y轴标签
    plt.colorbar(scatter, label='Class')  # 添加颜色条
    plt.grid(True, alpha=0.3)  # 添加半透明网格
    plt.show()


#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/12/6 23:15:46

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import laok
import cv2
from ultralytics import YOLO
# ===============================================================================
r'''
'''
# ===============================================================================
__all__ =['YoloDet', 'YoloSeg', ]

class YoloDet:
    def __init__(self, model_file, cls_file=None):
        self.setModel(model_file)
        self.setClsFile(cls_file)

    def setClsFile(self, cls_file):
        '''加载 名字索引文件
        '''
        self.nameIdx = laok.NameIndexFile(cls_file)

    def setModel(self, model_file):
        '''设置模型文件 .pt
        '''
        self.model = YOLO(model_file)

    def predict(self, img, conf_threshold=0.3, verbose=False):
        '''执行推理  cv2格式
        '''
        results = self.model.predict(img, verbose=verbose)
        # laok.dump(self.model.predictor)
        result = results[0]  # 单个图片, batch=1

        res = []
        for i, box in enumerate(result.boxes):
            conf = box.conf.cpu().item()
            if conf < conf_threshold:
                continue
            cls = box.cls.cpu().item()
            x1, y1, x2, y2 = box.xyxy.cpu().flatten().tolist()
            name = self.nameIdx.getName(cls)
            res.append({
                "conf": conf,
                "cls": int(cls),
                "name": name,
                "rect": [x1, y1, x2-x1, y2-y1]
                })
        return res


class YoloSeg:
    def __init__(self, model_file, cls_file=None):
        self.setModel(model_file)
        self.setClsFile(cls_file)

    def setClsFile(self, cls_file):
        '''加载 名字索引文件
        '''
        self.nameIdx = laok.NameIndexFile(cls_file)

    def setModel(self, model_file):
        '''设置模型文件 .pt
        '''
        self.model = YOLO(model_file)

    def predict(self, img, conf_threshold=0.3, verbose=False):
        '''执行推理  cv2格式
        '''
        results = self.model.predict(img, verbose=verbose)
        result = results[0]  # 单个图片, batch=1

        res = []
        for i, box in enumerate(result.boxes):
            conf = box.conf.cpu().item()
            if conf < conf_threshold:
                continue

            cls = box.cls.cpu().item()
            name = self.nameIdx.getName(cls)

            x1, y1, x2, y2 = box.xyxy.cpu().flatten().tolist()

            mask = result.masks[i].data[0].cpu().numpy()

            im0_shape = result.orig_shape
            mask = cv2.resize(mask, (im0_shape[1], im0_shape[0]), interpolation=cv2.INTER_LINEAR)  # INTER_CUBIC would be better

            res.append({
                "conf": conf,
                "cls": int(cls),
                "name": name,
                "rect": [x1, y1, x2-x1, y2-y1],
                "mask": mask,
                })
        return res


#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/8/2 22:28:18

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
from sklearn import linear_model
from sklearn import svm
# ===============================================================================
r'''
'''
# ===============================================================================
__all__ = [
 'RegressorARD',
 'RegressorBayesianRidge',
 'RegressorElasticNet',
 'RegressorElasticNetCV',
 'RegressorHuber',
 'RegressorLasso',
 'RegressorLassoCV',
 'RegressorLassoLars',
 'RegressorLassoLarsCV',
 'RegressorLinear',
 'RegressorLinearSVR',
 'RegressorMultiTaskElasticNet',
 'RegressorMultiTaskElasticNetCV',
 'RegressorMultiTaskLasso',
 'RegressorMultiTaskLassoCV',
 'RegressorNuSVR',
 'RegressorPassiveAggressive',
 'RegressorRANSAC',
 'RegressorRidge',
 'RegressorRidgeCV',
 'RegressorSGD',
 'RegressorSVR',
 'RegressorTheilSen']


# linear_model
RegressorLinear = linear_model.LinearRegression

RegressorRidge = linear_model.Ridge
RegressorRidgeCV = linear_model.RidgeCV

RegressorLasso = linear_model.Lasso
RegressorLassoCV = linear_model.LassoCV
RegressorMultiTaskLasso = linear_model.MultiTaskLasso
RegressorMultiTaskLassoCV = linear_model.MultiTaskLassoCV

RegressorElasticNet = linear_model.ElasticNet
RegressorElasticNetCV = linear_model.ElasticNetCV
RegressorMultiTaskElasticNet = linear_model.MultiTaskElasticNet
RegressorMultiTaskElasticNetCV = linear_model.MultiTaskElasticNetCV

RegressorLassoLars = linear_model.LassoLars
RegressorLassoLarsCV = linear_model.LassoLarsCV

RegressorBayesianRidge = linear_model.BayesianRidge

RegressorARD = linear_model.ARDRegression

RegressorSGD = linear_model.SGDRegressor

RegressorPassiveAggressive = linear_model.PassiveAggressiveRegressor

RegressorRANSAC = linear_model.RANSACRegressor

RegressorTheilSen = linear_model.TheilSenRegressor

RegressorHuber = linear_model.HuberRegressor

# svm
RegressorNuSVR = svm.NuSVR
RegressorSVR = svm.SVR
RegressorLinearSVR = svm.LinearSVR




#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022/10/17 19:59:14

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import pickle
# ===============================================================================
#
# ===============================================================================
__all__ = ['save_pickle_file', 'load_pickle_file']

def save_pickle_file(filename, obj, **kws):
    with open(filename, 'wb') as f:
        pickle.dump(obj, f, **kws)
    return True

def load_pickle_file(filename, **kws):
    with open(filename, 'rb') as f:
        return pickle.load(f, **kws)




#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022/10/17 20:30:40

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import torch
# ===============================================================================
r'''
'''
# ===============================================================================
__all__ = ['save_model', 'save_model_params', 'load_model', 'load_model_params', 'load_model_params_obj']

def save_model(model, filename):
    torch.save(model, filename)
    return model

def save_model_params(model, filename):
    torch.save(model.state_dict(), filename)
    return model

def load_model(filename, map_location=None):
    return torch.load(filename, map_location=map_location)

def load_model_params(model, filename, map_location=None):
    _obj = load_model(filename, map_location=map_location)
    return load_model_params_obj(model, _obj)

def load_model_params_obj(model, data):
    if isinstance(data, torch.nn.Module):
        state = data.state_dict()
    else:
        state = data

    # 加载匹配的部分
    _model_state = model.state_dict()
    part_load = {}
    for k,v in state.items():
        if k in _model_state and _model_state[k].shape == v.shape:
            part_load[k] = v
    _model_state.update(part_load)
    model.load_state_dict(_model_state)

    return model


#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022/10/16 10:30:01

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import time
from datetime import datetime
from .fpath import path_join, path_make_parent
# ===============================================================================
r'''
'''
# ===============================================================================
__all__ = ['PathGenSeq', 'PathGenTimed']

class _PathGenBase:
    def __init__(self, parent="", ext="", stem_pre="", stem_suf="", create_dir=False):
        self.setParent(parent)
        self.setExt(ext)
        self.setStemPre(stem_pre)
        self.setStemSuf(stem_suf)
        self.setCreateDir(create_dir)

    def setExt(self, ext):
        if ext:
            self._ext = ext if ext.startswith(".") else '.' + ext

    def setStemPre(self, stem_pre):
        self._stem_pre = stem_pre

    def setStemSuf(self, stem_suf):
        self._stem_suf = stem_suf

    def setParent(self, parent):
        self._parent = parent

    def setCreateDir(self, create_dir):
        self._create_dir = create_dir

    def _get_stem(self): # 需要覆盖该函数,实现序列化名字
        return ""

    def __next__(self):
        return self.next_path()

    def __iter__(self):
        return self

    def next_path(self):
        if self._create_dir:
            make_parent = True
            self._create_dir = False
        else:
            make_parent = False
        pth = path_join(self._parent, self._stem_pre + self._get_stem() + self._stem_suf + self._ext,
                        make_parent=make_parent)
        return pth

    def __call__(self):
        return self.next_path()


class PathGenSeq(_PathGenBase):
    def __init__(self, parent="", ext="", start=0, width=8, stem_pre="", stem_suf="", create_dir=False):
        super().__init__(parent=parent, ext=ext, stem_pre=stem_pre, stem_suf=stem_suf, create_dir=create_dir)
        self._start = start
        self._width = width

    def _get_stem(self):
        fmt = "{:0" + str(self._width) + "}"
        fname = fmt.format(self._start)
        self._start += 1
        return fname

class PathGenTimed(_PathGenBase):
    def __init__(self, parent="", ext="", time_fmt="%Y%layer%d_%H%M%S", stem_pre="", stem_suf="", create_dir=False):
        super().__init__(parent=parent, ext=ext, stem_pre=stem_pre, stem_suf=stem_suf, create_dir=create_dir)
        self._last_tick = 0
        self._last_inc = 0
        self._time_fmt = time_fmt

    def _get_stem(self):
        cur_time = int(time.time())
        dt = datetime.fromtimestamp(cur_time)
        if self._last_tick != cur_time:
            self._last_inc = 0
            self._last_tick = cur_time
            return dt.strftime(self._time_fmt)
        else:
            self._last_inc += 1
            self._last_tick = cur_time
            return dt.strftime(self._time_fmt) + "-" +str(self._last_inc)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/8/28 14:29:40

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
from .fwalk import *
#===============================================================================
r'''
'''
#===============================================================================
__all__ = ['suffix_image_list', 'suffix_dll_list', 'suffix_video_list',
           'image_cur', 'image_under',
           'video_cur', 'video_under',
           'dll_cur', 'dll_under',
           ]

def suffix_image_list():
    return [".bmp",".dib",	                # Windows位图文件
            ".jpeg",".jpg",".jpe",".jfif",	#JPEG文件
            ".png",	                        #便携式网络图片
            ".pbm",".pgm",".ppm",".pxm",".pnm",".pfm",	#便携式图像格式
            ".sr",".ras",	        #Sun rasters
            ".tiff",".tif",	        #TIFF文件
            ".exr",	                #OpenEXR HDR 图片
            ".hdr",
            ".pic",
            ".jp2",                 #JPEG 2000 图片
            ".webp"                 #WebP格式
            ]

def suffix_dll_list():
    return [".dll",".kll", ".so", ".pyd"]

def suffix_video_list():
    return [
        ".avi", # 音频视频交错(Audio Video Interleaved)AVI这个由微软公司发布的视频格式，在视频领域可以说是最悠久的格式之一。AVI格式调用方便、图像质量好，压缩标准可任意选择，是应用最广泛、也是应用时间最长的格式之一
        ".mpeg", ".mpg",  # Motion Picture Experts Group 的缩写。这类格式包括了MPEG-1,MPEG-2和MPEG-4在内的多种视频格式
        ".mov",  # MOV即QuickTime影片格式，它是Apple公司开发的一种音频、视频文件格式，用于存储常用数字媒体类型
        ".rm", ".ram",  # RealVideo视频格式是网络上的常用格式，对网络带宽要求比较低，能实现快速播放，但其视频画质没有其他格式视频高
        ".rmvb",  # RMVB是一种视频文件格式，其中的VB指Variable Bit Rate（可变比特率）。较上一代RM格式画面要清晰很多，原因是降低了静态画面下的比特率
        ".flv", ".swf",  # FLV是FLASH VIDEO的简称，FLV流媒体格式是一种新的视频格式。由于它形成的文件极小、加载速度极快，使得网络观看视频文件成为可能，
        ".mp4",  # MP4是一套用于音频、视频信息的压缩编码标准,MPEG-4格式的主要用途在于网上流、光盘、语音发送（视频电话），以及电视广播
        ".3gp",  # 3GP是第三代合作伙伴项目计划.为3G UMTS多媒体服务定义的一种多媒体容器格式，主要应用于3G移动电话，但也能在一些2G和4G手机上播放
        ".asf",  # (Advanced Streaming format高级流格式)。ASF 是MICROSOFT 为了和的Real player 竞争而发展出来的一种可以直接在网上观看视频节目的文件压缩格式
    ]

def image_cur(dir_names, suffixes = None):
    if suffixes is None:
        suffixes = suffix_image_list()
    yield from files_cur(dir_names=dir_names, suffixes=suffixes)

def image_under(dir_names, suffixes = None):
    if suffixes is None:
        suffixes = suffix_image_list()
    yield from files_under(dir_names=dir_names, suffixes=suffixes)

def video_cur(dir_names, suffixes = None):
    if suffixes is None:
        suffixes = suffix_video_list()
    yield from files_cur(dir_names=dir_names, suffixes=suffixes)

def video_under(dir_names, suffixes = None):
    if suffixes is None:
        suffixes = suffix_video_list()
    yield from files_under(dir_names=dir_names, suffixes=suffixes)

def dll_cur(dir_names, suffixes = None):
    if suffixes is None:
        suffixes = suffix_dll_list()
    yield from files_cur(dir_names=dir_names, suffixes=suffixes)

def dll_under(dir_names, suffixes = None):
    if suffixes is None:
        suffixes = suffix_dll_list()
    yield from files_under(dir_names=dir_names, suffixes=suffixes)


#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/7/11 20:36:06

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import cv2
from laok.base.colors import color_rand
from .info import image_size, image_center
from .util import keep_int32
# ===============================================================================
r'''

绘制图元:
    line
    circle
    rectangle
    ellipse
    putText

常见参数:
    img：您要绘制形状的图像
    color：形状的颜色。
        对于BGR，将其作为元组传递，例如：(255,0,0)对于蓝色。
        对于灰度，只需传递标量值即可。
    thickness：线或圆等的粗细。
        如果对闭合图形（如圆）传递-1 ，它将填充形状。默认厚度= 1
    lineType：线的类型，是否为8连接线，抗锯齿线等。
        默认情况下，为8连接线。**cv.LINE_AA**给出了抗锯齿的线条，看起来非常适合曲线。
'''
# ===============================================================================
__all__ = [
            'draw_point',
            'draw_line', 'draw_polylines',
            'draw_rect',
            'draw_circle', 'draw_ellipse', 'draw_arc',
            'draw_text',
            'draw_contour', 'draw_contours',
            'draw_det_rect',
            'draw_img_mask',
            'draw_img_seg'
]

def draw_point(img, pt=None, size=1, color=(255,255,255)):
    if pt is None: #默认使用中心点
        pt = image_center(img)
    return cv2.circle(img, center=keep_int32(pt), radius=int(size), color=color, thickness=-1, lineType=cv2.LINE_AA)


def draw_line(img, pt1, pt2, color=(255,255,255), thickness=1, lineType=cv2.LINE_AA):
    return cv2.line(img, pt1=keep_int32(pt1), pt2=keep_int32(pt2), color=color, thickness=thickness, lineType=lineType)

def draw_polylines(img, pts, isClosed=False, color=(255,255,255), thickness=1, lineType=cv2.LINE_AA):
    '''
    pts可以设置为 点列表: [[10,5],[20,30],[70,20],[50,10]]
    '''
    return cv2.polylines(img, pts=[ keep_int32(pts) ],
                         isClosed=isClosed, color=color, thickness=thickness, lineType=lineType)

def draw_rect(img, rect=None, color=(255,255,255), thickness=1, lineType=cv2.LINE_AA):
    if rect is None: #默认是图片的中心1/2大小
        w, h = image_size(img)
        rect = (w//4, h//4, w//2, h//2)
    return cv2.rectangle(img, rec=keep_int32(rect), color=color, thickness=thickness, lineType=lineType)

def draw_circle(img, center=None, radius= 5, color=(255,255,255), thickness=1, lineType=cv2.LINE_AA):
    if center is None: #默认使用中心点
        center = image_center(img)
    return cv2.circle(img, center=keep_int32(center), radius=int(radius), color=color, thickness=thickness, lineType=lineType)

def draw_ellipse(img, center=None, axes=(10, 5), angle=0,
                 color=(255,255,255), thickness=1, lineType=cv2.LINE_AA):
    if center is None:
        center = image_center(img)
    return cv2.ellipse(img, center= keep_int32(center), axes=keep_int32(axes), angle=angle,
                startAngle=0, endAngle=360,
                color=color, thickness=thickness, lineType=lineType)

def draw_arc(img, center=None, axes=(10, 5), angle=0, startAngle=0, endAngle=90,
             color=(255,255,255), thickness=1, lineType=cv2.LINE_AA):
    if center is None:
        center = image_center(img)
    return cv2.ellipse(img, center=center, axes=axes, angle=angle,
                startAngle=startAngle, endAngle=endAngle,
                color=color, thickness=thickness, lineType=lineType)

def draw_text(img, text='lk', pos=None, fontFace=cv2.FONT_HERSHEY_SIMPLEX, fontScale=1.0,
              color=(255,255,255), thickness=2, lineType=cv2.LINE_AA):
    if pos is None:
        pos = image_center(img)
    return cv2.putText(img, text=text, org=keep_int32(pos), fontFace=fontFace, fontScale=fontScale,
                color=color, thickness=thickness, lineType=lineType)

def draw_contour(img, contour, color=(255,255,255), thickness=2, lineType=cv2.LINE_AA):
    return cv2.drawContours(img, [contour], contourIdx=-1, color=color, thickness=thickness, lineType=lineType)

def draw_contours(img, contours, color=(255,255,255), thickness=2, lineType=cv2.LINE_AA):
    if color is None:
        for i, contour in enumerate(contours):
            draw_contour(img, contour, color=color_rand(i), thickness=thickness, lineType=lineType)
    else:
        return cv2.drawContours(img, contours, contourIdx=-1, color=color, thickness=thickness, lineType=lineType)

def draw_det_rect(img, rect, conf, name, color=(0,0,255), thickness=2, fontScale=0.5, lineType=cv2.LINE_AA):
    # Draw a bounding box.
    left, top = rect[0:2]
    draw_rect(img, rect=rect, color=color, thickness=thickness, lineType=lineType)

    label = '{} {:.2f}'.format(name, conf)
    # Display the label at the top of the bounding box
    labelSize, baseLine = cv2.getTextSize(label, cv2.FONT_HERSHEY_SIMPLEX, fontScale=fontScale, thickness=thickness)
    top = max(top, labelSize[1])
    inc_y = round(1.0 * labelSize[1])
    width = 1.0 * labelSize[0]
    height = baseLine + inc_y
    draw_rect(img, [left, top-inc_y, width, height], color=color, thickness=cv2.FILLED)
    draw_text(img, label, pos=(left, top), fontFace=cv2.FONT_HERSHEY_SIMPLEX, fontScale=fontScale, color=(255, 255, 125), thickness=thickness)


def draw_img_seg(img, seg_img):
    pass

def draw_img_mask(img, mask_img, color=(0,0,255)):
    segmented_image = cv2.bitwise_and(img, img, mask=mask_img)




#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/8/3 22:56:24

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
from sklearn import preprocessing
# ===============================================================================
r'''
'''
# ===============================================================================
__all__ = ['TransStandardScaler',

           ]

# function
TransFunction = preprocessing.FunctionTransformer

# data
TransBinarizer = preprocessing.Binarizer
TransKernelCenterer = preprocessing.KernelCenterer
TransMaxAbsScaler = preprocessing.MaxAbsScaler
TransNormalizer = preprocessing.Normalizer
TransRobustScaler = preprocessing.RobustScaler
TransStandardScaler = preprocessing.StandardScaler
TransQuantile = preprocessing.QuantileTransformer
TransPower = preprocessing.PowerTransformer

trans_add_dummy = preprocessing.add_dummy_feature
trans_binarize = preprocessing.binarize
trans_normalize = preprocessing.normalize
trans_scale = preprocessing.scale
trans_robust_scale = preprocessing.robust_scale
trans_maxabs_scale = preprocessing.maxabs_scale
trans_minmax_scale = preprocessing.minmax_scale
trans_quantile = preprocessing.quantile_transform
trans_power = preprocessing.power_transform

# encode
TransEncoderOneHot = preprocessing.OneHotEncoder
TransEncoderOrdinal = preprocessing.OrdinalEncoder

# label
TransLabelBinarizer = preprocessing.LabelBinarizer
TransLabelEncoder = preprocessing.LabelEncoder
TransLabelMultiEncoder = preprocessing.MultiLabelBinarizer

trans_label_binarize = preprocessing.label_binarize

# discretization
TransKBinsDiscretizer = preprocessing.KBinsDiscretizer

# polynomial
TransPolyFeatures = preprocessing.PolynomialFeatures
TransPolySpline = preprocessing.SplineTransformer




#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/5/4 20:27:02
@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import os
from ultralytics import YOLO
import laok
from laok.ext.cv2_.io import fix_cv_read
from laok.ext.yaml_ import save_yaml_file
from laok import log_info

from .export import export_onnx
#===============================================================================
r'''支持 yolo训练的脚本
需要 文件夹 root_dir下:
                ---- xxx.pt
                ---- xxx.names
                ---- Image/
imgsz = (height, width)
'''
#===============================================================================
__all__ = ['train_det', 'train_cls', 'train_seg', 'train_obb', 'train_pose']


def _train(root_dir, train_path='Image', val_path='Image', test_path=None, model_file=None, **kws):
    log_info(f'root_dir= {root_dir}')
    fix_cv_read()

    with laok.workdir_scope(root_dir):
        if model_file is None:
            model_file = laok.path_search_cur_ext('.', ".pt")

        # ### 删除.cache
        # for fname in laok.path_search_under_ext(train_path, '.cache'):
        #     log_info(f'delete file:{fname}')
        #     laok.file_delete(fname)

        ### 生成 yaml
        ds = laok.Dict()
        ds.path = os.getcwd()
        ds.train = train_path
        ds.val = val_path
        ds.test = test_path
        if 'kpt_shape' in kws:
            ds.kpt_shape = kws.pop('kpt_shape')
        if 'flip_idx' in kws:
            ds.flip_idx = kws.pop('flip_idx')
        names_file = laok.path_search_cur_ext('.', ".names")
        ds.names = list(line.split(":")[0] for line in laok.file_read_lines(names_file))
        data_file = laok.path_replace_ext(model_file, ".yaml")
        save_yaml_file(data_file, ds, indent=4, encoding='utf8')

        ### 训练
        model = YOLO(model_file)
        kws['data'] = data_file
        kws.setdefault('save_txt', True)
        kws.setdefault('save_conf', True)
        kws.setdefault('save_crop', True)
        kws.setdefault('amp', False)
        kws.setdefault('degrees', 3)
        if laok.is_windows():
            kws.setdefault('workers', 0)
        model.train(**kws)

        # 导出onnx
        dst_file = kws.pop('dst_file', None)
        best_model = laok.path_search_under_file(str(model.trainer.wdir), 'best.pt')
        export_onnx(best_model, imgsz=kws['imgsz'], dst_file=dst_file)


def _train_cls(root_dir, image_path='Image', model_file=None, **kws):
    log_info(f'root_dir= {root_dir}')
    fix_cv_read()
    with laok.workdir_scope(root_dir):
        if model_file is None:
            model_file = laok.path_search_cur_ext('.', ".pt")

        from torchvision.datasets.folder import find_classes
        data_dir = image_path
        if os.path.exists( os.path.join(image_path, 'train') ):
            data_dir = os.path.join(image_path, 'train')
        classes, class_to_idx = find_classes(data_dir)
        laok.file_write_lines('cls.names', classes, encoding='gbk')

        ### 训练
        model = YOLO(model_file)
        kws['data'] = image_path
        kws.setdefault('save_txt', True)
        kws.setdefault('save_conf', True)
        kws.setdefault('save_crop', True)
        kws.setdefault('amp', False)
        kws.setdefault('degrees', 3)
        if laok.is_windows():
            kws.setdefault('workers', 0)
        model.train(**kws)

        # 导出onnx
        dst_file = kws.pop('dst_file', None)
        best_model = laok.path_search_under_file(str(model.trainer.wdir), 'best.pt')
        export_onnx(best_model, imgsz=kws['imgsz'], dst_file=dst_file)



def train_det(root_dir, imgsz=(640, 640), epochs=100, batch=16, **kws):
    _train(root_dir=root_dir, task='detect', imgsz=imgsz, epochs=epochs, batch=batch, **kws)

def train_seg(root_dir, imgsz=(640, 640), epochs=100, batch=16, **kws):
    _train(root_dir=root_dir, task='segment', imgsz=imgsz, epochs=epochs, batch=batch, **kws)


def train_cls(root_dir, imgsz=(640, 640), epochs=100, batch=16, **kws):
    _train_cls(root_dir=root_dir, task='classify', imgsz=imgsz, epochs=epochs, batch=batch, **kws)


def train_obb(root_dir, imgsz=(640, 640), epochs=100, batch=16, **kws):
    _train(root_dir=root_dir, task='obb', imgsz=imgsz, epochs=epochs, batch=batch, **kws)


def train_pose(root_dir, kpt_shape, imgsz=(640, 640), epochs=100, batch=16, **kws):
    _train(root_dir=root_dir, kpt_shape=kpt_shape, task='pose', imgsz=imgsz, epochs=epochs, batch=batch, **kws)




#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2021/3/17 21:11:08

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import time, sys
from functools import wraps
from collections import OrderedDict

from contextlib import contextmanager
#===============================================================================
# 
#===============================================================================
__all__ = ['time_sleep', 'time_deco', 'time_ctx', 'Timer', 'TimerLabel']

time_sleep = time.sleep

def time_deco(stream = None):
    ''' 装饰器,记录时间 '''
    def _w1(func):
        @wraps(func)
        def _w2(*args, **kwargs):
            start = time.time()
            res = func(*args, **kwargs)
            stop = time.time()
            _stream = stream or sys.stdout
            _stream.write(f"[{ func.__name__ }] use time [{ 1000*(stop-start) }(ms)]\n")
            return res
        return _w2
    return _w1


@contextmanager
def time_ctx(msg=None, stream=None):
    '''上下文计时器'''
    t1 = time.time()
    yield
    t2 = time.time()
    eps = (t2-t1)*1000

    _stream = stream or sys.stdout
    if msg is not None:
        _stream.write(f'{msg} ')
    _stream.write(f'time:{eps}(ms)\n')


class Timer:
    def __init__(self):
        self.restart()

    def restart(self):
        self._t1 = time.time()

    def elapse(self): #单位 (s)
        return time.time() - self._t1


class TimerLabel:
    def __init__(self):
        self._t1 = OrderedDict()
        self._t2 = OrderedDict()
        self._enabled = True
        self._total_count = None
        self._cur_count = 0

    def setEnabled(self, val):
        self._enabled = val

    def setReportCount(self, cnt):
        self._total_count = cnt
        self._cur_count = 0

    def start(self, name):
        if self._enabled:
            self._t1[name] = time.time()

    def end(self, name):
        if self._enabled:
            self._t2[name] = time.time()

    def report(self, stream = None):
        if not self._enabled:
            return 
        
        if self._total_count is not None:
            if self._cur_count >= self._total_count:
                return
            self._cur_count += 1

        _stream = stream or sys.stdout
        
        curt = time.time()
        _stream.write("\n***** time summary *****\n")
        for name,t1 in self._t1.items():
            t2 = self._t2.get(name, curt)
            _stream.write(f'"{name}" time use: {t2-t1}(s)\n')

if __name__ == "__main__":
    times = TimerLabel()
    times.setReportCount(2)

    for i in range(5):
        times.start('it1')
        time.sleep(1)
        times.start('it2')
        time.sleep(3)
        times.report()


#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/7/11 22:42:30

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import numpy as np
# ===============================================================================
r'''
'''
# ===============================================================================
__all__ = ['keep_uint8', 'keep_int32', 'new_color', 'new_gray']

def keep_uint8(data):
    return np.array(data, dtype=np.uint8)

def keep_int32(data):
    return np.array(data, dtype=np.int32)

def new_color(width=512, height=512, color=(0,0,0), dtype=np.uint8):
    return np.full((height, width, 3), fill_value=color, dtype=dtype)

def new_gray(width=512, height=512, value=0, dtype=np.uint8):
    return np.full( (height, width), fill_value=value, dtype=dtype)




#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/8/28 08:40:06

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import laok
from laok.ext import lxml_
from laok.ext import cv2_
from laok.ext import imagesize_
#===============================================================================
r'''
'''
#===============================================================================
__all__ = ['voc_to_yolo', 'yolo_to_voc', 'voc_fix_resize', 'voc_empty_remove',
           'voc_extract_dataset',
           'voc_extract_roi_dataset']

def _read_xml(xml_file):
    vocs = {}
    root = lxml_.load_xml(xml_file)
    size = root.find('size')
    w = size.find('width').text
    h = size.find('height').text
    vocs['width'] = int(w)
    vocs['height'] = int(h)
    vocs['filename'] = root.find('filename').text
    obj_arr = []
    for obj in root.findall('object'):
        bndbox = obj.find('bndbox')
        xmin, xmax, ymin, ymax = bndbox.find('xmin').text, bndbox.find('xmax').text, \
            bndbox.find('ymin').text, bndbox.find('ymax').text
        obj_arr.append({
            'name': obj.find('name').text,
            'xmin': float(xmin),
            'xmax': float(xmax),
            'ymin': float(ymin),
            'ymax': float(ymax),
        })
    vocs['object'] = obj_arr
    return vocs

def _save_xml(xml_file, vocs):
    E = lxml_.get_maker()
    annotation = E.annotation(
        E.size(E.width(vocs['width']), E.height(vocs['height'])),
        E.filename(vocs['filename']),
    )
    for voc in vocs['object']:
        obj = E.object(
            E.name(voc['name']),
            E.bndbox( E.xmin(voc['xmin']), E.ymin(voc['ymin']), E.xmax(voc['xmax']), E.ymax(voc['ymax']) ),
        )
        annotation.append(obj)
    lxml_.save_xml(xml_file, annotation)

def _read_txt(txt_file):
    yolos = []
    for line in laok.file_read_lines(txt_file, strip=True, skip_empty=True):
        idx, cx, cy, rw, rh = line.split()
        data = int(idx), float(cx), float(cy), float(rw), float(rh)
        yolos.append( data )
    return yolos

def _box_to_voc(imgsz, cx, cy, rw, rh):
    cx, cy, rw, rh = float(cx), float(cy), float(rw), float(rh)
    w,h = imgsz
    w2 = w * rw
    h2 = h * rh
    x1 = cx * w - w2/2
    y1 = cy * h - h2/2
    x2 = x1 + w2
    y2 = y1 + h2
    return round(x1), round(y1), round(x2), round(y2)

def _box_to_yolo(w, h, xmin, ymin, xmax, ymax):
    dw = 1. / w
    dh = 1. / h
    cx = ( (xmin + xmax) / 2.0 )*dw
    cy = ( (ymin + ymax) / 2.0 )*dh
    rw = (xmax - xmin) * dw
    rh = (ymax - ymin) * dh
    return cx, cy, rw, rh

def _data_to_yolo(xml_file, nameIdx):
    txt_file = laok.path_replace_ext(xml_file, '.txt')
    with open(txt_file, mode='w', encoding='utf8') as f:
        vocs = _read_xml(xml_file)
        w = vocs['width']
        h = vocs['height']
        for obj in vocs['object']:
            name = obj['name']
            idx = nameIdx.getIndex(name)
            assert idx != -1 and f"{xml_file} can't find [{name}]"
            cx, cy, rw, rh = _box_to_yolo(w, h,
                                        obj['xmin'], obj['ymin'],
                                        obj['xmax'], obj['ymax'])
            f.write(f"{idx} {cx} {cy} {rw} {rh}\n")

def _data_to_voc(img_file, nameIdx):
    txt_file = laok.path_replace_ext(img_file, ".txt")
    if not laok.path_exist(txt_file):
        return

    imgsz = imagesize_.get_size(img_file)
    w, h = imgsz

    vocs = {}
    vocs['width'] = w
    vocs['height'] = h
    vocs['filename'] = laok.path_basename(img_file)
    obj_arr = []
    vocs['object'] = obj_arr
    for idx, cx, cy, rw, rh in _read_txt(txt_file):
        name = nameIdx.getName(idx)
        assert name and f"{txt_file} can't find [{idx}]"
        x1, y1, x2, y2 = _box_to_voc(imgsz, cx, cy, rw, rh)
        obj_arr.append({
            'name':name,
            'xmin': x1,
            'ymin': y1,
            'xmax': x2,
            'ymax': y2,
        })
    _save_xml(txt_file, vocs)

def _voc_fix(img_file):
    xml_file = laok.path_replace_ext(img_file, ".xml")
    if not laok.path_exist(xml_file):
        return

    imgsz = imagesize_.get_size(img_file)
    w, h = imgsz
    vocs = _read_xml(xml_file)

    w2, h2 = vocs['width'], vocs['height']
    if w2 == w and h2 == h:
        return

    dw = w / w2
    dh = h / h2
    vocs['width'] = w
    vocs['height'] = h
    for voc in vocs['object']:
        voc['xmin'] = int(voc['xmin'] * dw)
        voc['xmax'] = int(voc['xmax'] * dw)
        voc['ymin'] = int(voc['ymin'] * dh)
        voc['ymax'] = int(voc['ymax'] * dh)
    _save_xml(xml_file, vocs)

def _voc_remove_empty(img_file):
    #如果没有xml或则txt,则删除img_file
    txt_file = laok.path_replace_ext(img_file, 'txt')
    xml_file = laok.path_replace_ext(img_file, 'xml')

    if laok.path_exist(xml_file):
        vocs = _read_xml(xml_file)
        if len(vocs['object']) == 0:
            laok.file_delete(img_file)
            laok.file_delete(xml_file)
            laok.file_delete(txt_file)
            print(f'empty xml label, del: {img_file}')
    elif laok.path_exist(txt_file):
        yolos = _read_txt(txt_file)
        if len(yolos) == 0:
            laok.file_delete(img_file)
            laok.file_delete(txt_file)
            print(f'empty txt label, del: {img_file}')
    else:
        laok.file_delete(img_file)
        print(f'no label file, del: {img_file}')


def voc_to_yolo(data_dir, name_file):
    nameIdx = laok.NameIndexFile(name_file)
    for xml_file in laok.files_under(data_dir, ".xml"):
        _data_to_yolo(xml_file, nameIdx)

def yolo_to_voc(data_dir, name_file):
    nameIdx = laok.NameIndexFile(name_file)
    for img_file in laok.image_under(data_dir):
        _data_to_voc(img_file, nameIdx)


def voc_fix_resize(data_dir):
    for img_file in laok.image_under(data_dir):
        _voc_fix(img_file)

def voc_empty_remove(data_dir):
    for img_file in laok.image_under(data_dir):
        _voc_remove_empty(img_file)

def voc_extract_dataset(data_dir, subset_dir, object_names):
    data_dir = laok.path_unix(data_dir)
    laok.path_make(subset_dir)

    # 保存类别文件
    cls_name = laok.path_basename(subset_dir)
    cls_file_name = laok.path_join(subset_dir, cls_name + ".names")
    laok.file_write_lines(cls_file_name, object_names)

    # 生成Image文件夹
    subset_dir = laok.path_join(subset_dir, 'Image')
    laok.path_make(subset_dir)

    # 保存 Image + Xml
    for img_file in laok.image_under(data_dir):

        #读取 voc 数据
        xml_file = laok.path_replace_ext(img_file, '.xml')
        if not laok.path_exist(xml_file):
            continue
        data = _read_xml(xml_file)
        data['object'] = [ obj for obj in data['object'] if obj['name'] in object_names]

        new_img_file = img_file.replace(data_dir, subset_dir)
        new_xml_file = laok.path_replace_ext(new_img_file, '.xml')
        laok.path_make_parent(new_img_file)
        _save_xml(new_xml_file, data)

        #复制图像
        laok.copy(img_file, new_img_file)

def voc_extract_roi_dataset(data_dir, subset_dir, object_roi, object_names):
    data_dir = laok.path_unix(data_dir)
    laok.path_make(subset_dir)

    # 保存类别文件
    cls_name = laok.path_basename(subset_dir)
    cls_file_name = laok.path_join(subset_dir, cls_name + ".names")
    laok.file_write_lines(cls_file_name, object_names)

    # 生成Image文件夹
    subset_dir = laok.path_join(subset_dir, 'Image')
    laok.path_make(subset_dir)

    # 保存 Image + Xml
    for img_file in laok.image_under(data_dir):
        #读取 voc 数据
        xml_file = laok.path_replace_ext(img_file, '.xml')
        if not laok.path_exist(xml_file):
            continue
        data = _read_xml(xml_file)

        # 计算所有 roi
        rois = {}
        for obj in data['object']:
            if obj['name'] == object_roi:
                x1, y1, x2, y2 = obj['xmin'], obj['ymin'], obj['xmax'], obj['ymax']
                roi = (int(x1), int(y1), int(x2), int(y2))
                rois[roi] = []

        # 计算所有的子图像
        for obj in data['object']:
            if obj['name'] not in object_names:
                continue

            x1, y1, x2, y2 = obj['xmin'], obj['ymin'], obj['xmax'], obj['ymax']
            for roi2, objList in rois.items():
                a1, b1, a2, b2 = roi2
                if x1 >= a1 and y1 >= b1 and x2 <= a2 and y2 <= b2:
                    objList.append(obj)
                    break


        # print(img_file)
        # print(new_img_file)
        # print(new_xml_file)
        # print(img)
        # print(rois)
        #复制图像
        img = cv2_.read_color(img_file)
        new_img_file = img_file.replace(data_dir, subset_dir)
        laok.path_make_parent(new_img_file)

        for idx, (roi, objList) in enumerate(rois.items()):
            if len(objList) == 0:
                continue
            x1, y1, x2, y2 = roi
            for obj in objList:
                obj['xmin'] = int(obj['xmin'] - x1)
                obj['ymin'] = int(obj['ymin'] - y1)
                obj['xmax'] = int(obj['xmax'] - x1)
                obj['ymax'] = int(obj['ymax'] - y1)
            data['object'] = objList
            data['width'] = x2 - x1
            data['height'] = y2 - y1

            if len(rois) == 1:
                cv2_.write_image(new_img_file, img[y1:y2, x1:x2])
                data['filename'] = laok.path_basename(new_img_file)
                new_xml_file = laok.path_replace_ext(new_img_file, '.xml')
                _save_xml(new_xml_file, data)
                break

            new_img_file = laok.path_deco_stem(new_img_file, "", "--{}".format(idx))
            cv2_.write_image(new_img_file, img[y1:y2, x1:x2])
            data['filename'] = laok.path_basename(new_img_file)
            new_xml_file = laok.path_replace_ext(new_img_file, '.xml')
            _save_xml(new_xml_file, data)


if __name__ == '__main__':
    data_dir = r'D:\proj\东方瑞威-货车异常\data\train\人孔螺丝'
    subset_dir = r'D:\proj\东方瑞威-货车异常\data\train\tankcap'
    object_roi = 'tankcap'
    object_names = ['rkls']
    voc_extract_roi_dataset(data_dir, subset_dir, object_roi, object_names)




#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/7/14 10:13:56

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
from PyQt5.QtWidgets import QApplication, QWidget
import sys, inspect
# ===============================================================================
r'''
'''
# ===============================================================================
__all__ = ['run_widget']

def run_widget(widget = None, init_func = None, clear_func = None, *args, **kwargs):

    try:
        app = QApplication(sys.argv)

        if init_func:
            init_func(app)

        win = widget
        if inspect.isclass(widget) or inspect.isfunction(widget):
            win = widget(*args, **kwargs)

        if isinstance(win, QWidget):
            win.show()

        ret = app.exec_()

        if clear_func:
            clear_func(app)

        sys.exit(ret)

    except Exception as e:
        import traceback
        traceback.print_exc()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/11/27 20:59:23

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import laok
import imagesize
# ===============================================================================
r'''
'''
# ===============================================================================

def _box_to_yolo(w, h, xmin, ymin, xmax, ymax):
    dw = 1. / w
    dh = 1. / h
    cx = ( (xmin + xmax) / 2.0 )*dw
    cy = ( (ymin + ymax) / 2.0 )*dh
    rw = (xmax - xmin) * dw
    rh = (ymax - ymin) * dh
    return cx, cy, rw, rh

def rect_to_yolo(data_dir):
    for img_file in laok.image_under(data_dir):
        imgsz = imagesize.get(img_file)
        w, h = imgsz
        txt_file = laok.path_replace_ext(img_file, 'txt')
        need_cvt = True
        data_list = []
        for line in laok.file_read_lines(txt_file):
            tp, x1, y1, w1, h1 = line.split()
            x1, y1 = int(x1), int(y1)
            if x1 >= 1:
                need_cvt = False
                break
            x2, y2 = x1 + int(w1), y1 + int(h1)
            cx, cy, rw, rh = _box_to_yolo(w, h, x1, y1, x2, y2)
            data_list.append("{} {} {} {} {}".format(tp, cx, cy, rw, rh))

        if need_cvt:
            laok.file_write_lines(txt_file, data_list)





data_dir = r'C:\Users\k\Desktop\Image'
rect_to_yolo(data_dir)


#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/8/28 14:13:38

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import os
from contextlib import contextmanager
#===============================================================================
r'''
'''
#===============================================================================
__all__ = ['workdir_get', 'workdir_set', 'workdir_scope']
def workdir_get():
    return os.getcwd()

def workdir_set(fpath):
    os.chdir(fpath)

@contextmanager
def workdir_scope(dir):
    try:
        old_dir = workdir_get()
        yield os.chdir(dir)
    finally:
        workdir_set(old_dir)


#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/4/28 11:49:47
@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import numpy as np
#===============================================================================
r'''
'''
#===============================================================================
def farthest_point_sample_faster(pts: np.array, num: int) -> np.array:
    """
    Input:
        xyz: pointcloud data, [B, N, 3]
        npoint: number of samples
    Return:
        centroids: sampled pointcloud index, [B, npoint]
    """
    pc1 = np.expand_dims(pts, axis=0)   # 1, N, 3
    batchsize, npts, dim = pc1.shape
    # centroids 是当前点集中最远点的坐标集合, shape (B, N) -> (1, 300)  300时要采样的点数
    centroids = np.zeros((batchsize, num), dtype=np.long)
    # 距离的shape是 (B, ndataset) ndataset是输入点的数量
    distance = np.ones((batchsize, npts)) * 1e10
    # 初始化的最远点, 随机选取id, 如果batchsize不是1, 那就选取 batchsize个,
    farthest_id = np.random.randint(0, npts, (batchsize,), dtype=np.long)
    # batch_indices=[0,1,...,batchsize-1]
    batch_index = np.arange(batchsize)
    for i in range(num):
    # 更新第i个最远点的id, 这里时所有的batch都同时更新, farthest的维度和 centroids[:, i]的维度相同
        centroids[:, i] = farthest_id
    # 取出这个最远点的xyz坐标, 按维度分别取 batch\\点的id\\点的坐标, 然后view变换维度
        centro_pt = pc1[batch_index, farthest_id, :].reshape(batchsize, 1, 3)
    # 计算点集中的所有点到这个最远点的欧式距离
        # 等价于torch.sum((xyz - centroid) ** 2, 2)
        dist = np.sum((pc1 - centro_pt) ** 2, -1)
    # 更新distances，记录样本中每个点距离所有已出现的采样点的最小距离
        mask = dist < distance
    # 从更新后的distances矩阵中找出距离最远的点，作为最远点用于下一轮迭代
        distance[mask] = dist[mask]
        farthest_id = np.argmax(distance[batch_index])
    # 返回采样点的id
    return centroids


#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/8/11 12:35:20

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import laok
import shutil
import ultralytics
from functools import partial
# ===============================================================================
r'''
'''
# ===============================================================================
__all__ = ['export_onnx', 'export_onnx_yolo', 'export_onnx_yolo_sam', 'export_onnx_yolo_sam2',
           'export_onnx_yolo_sam_mobile', 'export_onnx_yolo_sam_fast',
           'export_onnx_yolo_nas', 'export_onnx_rt_detr',
           'export_onnx_yolo_world', 'export_onnx_yolo_e'
           ]

def export_onnx(src_file, imgsz, dst_file=None, cls_name='YOLO', **kws):
    '''
    opset = 12, dynamic=False, 用于支持 opencv-dnn
    '''
    cls = getattr(ultralytics, cls_name)
    model = cls(src_file)

    kws['imgsz'] = imgsz
    kws['format'] = kws.pop('format', 'onnx')
    kws['opset'] = kws.pop('opset', 12)
    kws['dynamic'] = kws.pop('dynamic', False)

    print('export args:', kws)
    model.export(**kws)
    if dst_file:
        onnx_model = laok.path_replace_ext(src_file, 'onnx')
        shutil.copy(onnx_model, dst_file)

export_onnx_yolo = export_onnx
export_onnx_yolo_sam = partial(export_onnx, cls_name='SAM')
export_onnx_yolo_sam2 = partial(export_onnx, cls_name='SAM')
export_onnx_yolo_sam_mobile = partial(export_onnx, cls_name='SAM')
export_onnx_yolo_sam_fast = partial(export_onnx, cls_name='FastSAM')
export_onnx_yolo_nas = partial(export_onnx, cls_name='NAS')
export_onnx_rt_detr = partial(export_onnx, cls_name='RTDETR', opset=16)
export_onnx_yolo_world = partial(export_onnx, cls_name='YOLOWorld')
export_onnx_yolo_e = partial(export_onnx, cls_name='YOLOE')






#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022/10/16 10:05:28

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
from .fpath import path_exist
# ===============================================================================
r'''
'''
# ===============================================================================
__all__ = ['file_read_text', 'file_write_text', 'file_read_lines', 'file_write_lines',
           'file_read_bin', 'file_read_chunk', 'file_write_bin',
           'file_text_reader', 'file_bin_reader',
           'file_bin_writer', 'file_text_writer',
           'file_list_merge'
           ]

def file_text_reader(fname, encoding='utf8'):
    return open(fname, mode='r', encoding=encoding)

def file_bin_reader(fname):
    return open(fname, mode='rb')

def file_text_writer(fname, encoding='utf8'):
    return open(fname, mode='w', encoding=encoding)

def file_bin_writer(fname):
    return open(fname, mode='wb')

def file_read_text(fname, encoding='utf8'):
    with open(fname, mode='r', encoding=encoding) as f:
        return f.read()
    return fname

def file_write_text(fname, text, encoding='utf8', skip_exist=False):
    if skip_exist and path_exist(fname):
        return fname
    with open(fname, mode='w', encoding=encoding) as f:
        f.write(text)
    return fname

def file_read_lines(fname, encoding='utf8', strip = True, skip_empty=False):
    with open(fname, mode='r', encoding=encoding) as f:
        for line in f:
            if strip:
                line = line.strip()
            if skip_empty:
                if not line :
                    continue
            yield line

def file_write_lines(fname, lines, encoding='utf8', line_append='\n', skip_exist=False):
    if skip_exist and path_exist(fname):
        return fname

    with open(fname, mode='w', encoding=encoding) as f:
        for line in lines:
            f.write(line + line_append)
    return fname

def file_read_bin(fname):
    with open(fname, mode='rb') as f:
        return f.read()
    return fname

def file_read_chunk(fname, buf_size=1e6):
    buf_size = int(buf_size)
    with open(fname, mode='rb') as f:
        while True:
            data = f.read(buf_size)
            if not data:
                break
            yield data

def file_write_bin(fname, data, skip_exist=False):
    if skip_exist and path_exist(fname):
        return fname
    with open(fname, mode='wb') as f:
        f.write(data)
    return fname

def file_list_merge(file_list, dst_file, buf_size=1e6):
    with file_bin_writer(dst_file) as df:
        for fname in file_list:
            for data in file_read_chunk(fname, buf_size=buf_size):
                df.write(data)
    return dst_file


""" Helper class and functions for loading KITTI objects

Author: Charles R. Qi
Date: September 2017
"""
import os
import sys
import numpy as np
import cv2
import utils as utils

class kitti_object(object):
    """Load and parse object data into a usable format."""

    def __init__(self, root_dir, split="training", args=None):
        """root_dir contains training and testing folders"""
        self.root_dir = root_dir
        self.split = split
        print(root_dir, split)
        self.split_dir = os.path.join(root_dir, split)

        if split == "training":
            self.num_samples = 7481
        elif split == "testing":
            self.num_samples = 7518
        else:
            print("Unknown split: %s" % (split))
            exit(-1)

        lidar_dir = "velodyne"
        depth_dir = "depth"
        pred_dir = "pred"
        if args is not None:
            lidar_dir = args.lidar
            depth_dir = args.depthdir
            pred_dir = args.preddir

        self.image_dir = os.path.join(self.split_dir, "image_2")
        self.label_dir = os.path.join(self.split_dir, "label_2")
        self.calib_dir = os.path.join(self.split_dir, "calib")

        self.depthpc_dir = os.path.join(self.split_dir, "depth_pc")
        self.lidar_dir = os.path.join(self.split_dir, lidar_dir)
        self.depth_dir = os.path.join(self.split_dir, depth_dir)
        self.pred_dir = os.path.join(self.split_dir, pred_dir)

    def __len__(self):
        return self.num_samples

    def get_image(self, idx):
        assert idx < self.num_samples
        img_filename = os.path.join(self.image_dir, "%06d.png" % (idx))
        return utils.load_image(img_filename)

    def get_lidar(self, idx, dtype=np.float32, n_vec=4):
        assert idx < self.num_samples
        lidar_filename = os.path.join(self.lidar_dir, "%06d.bin" % (idx))
        print(lidar_filename)
        return utils.load_velo_scan(lidar_filename, dtype, n_vec)

    def get_calibration(self, idx):
        assert idx < self.num_samples
        calib_filename = os.path.join(self.calib_dir, "%06d.txt" % (idx))
        return utils.Calibration(calib_filename)

    def get_label_objects(self, idx):
        assert idx < self.num_samples and self.split == "training"
        label_filename = os.path.join(self.label_dir, "%06d.txt" % (idx))
        return utils.read_label(label_filename)

    def get_pred_objects(self, idx):
        assert idx < self.num_samples
        pred_filename = os.path.join(self.pred_dir, "%06d.txt" % (idx))
        is_exist = os.path.exists(pred_filename)
        if is_exist:
            return utils.read_label(pred_filename)
        else:
            return None

    def get_depth(self, idx):
        assert idx < self.num_samples
        img_filename = os.path.join(self.depth_dir, "%06d.png" % (idx))
        return utils.load_depth(img_filename)

    def get_depth_image(self, idx):
        assert idx < self.num_samples
        img_filename = os.path.join(self.depth_dir, "%06d.png" % (idx))
        return utils.load_depth(img_filename)

    def get_depth_pc(self, idx):
        assert idx < self.num_samples
        lidar_filename = os.path.join(self.depthpc_dir, "%06d.bin" % (idx))
        is_exist = os.path.exists(lidar_filename)
        if is_exist:
            return utils.load_velo_scan(lidar_filename), is_exist
        else:
            return None, is_exist
        # print(lidar_filename, is_exist)
        # return utils.load_velo_scan(lidar_filename), is_exist

    def get_top_down(self, idx):
        pass

    def isexist_pred_objects(self, idx):
        assert idx < self.num_samples and self.split == "training"
        pred_filename = os.path.join(self.pred_dir, "%06d.txt" % (idx))
        return os.path.exists(pred_filename)

    def isexist_depth(self, idx):
        assert idx < self.num_samples and self.split == "training"
        depth_filename = os.path.join(self.depth_dir, "%06d.txt" % (idx))
        return os.path.exists(depth_filename)


class kitti_object_video(object):
    """ Load data for KITTI videos """

    def __init__(self, img_dir, lidar_dir, calib_dir):
        self.calib = utils.Calibration(calib_dir, from_video=True)
        self.img_dir = img_dir
        self.lidar_dir = lidar_dir
        self.img_filenames = sorted(
            [os.path.join(img_dir, filename) for filename in os.listdir(img_dir)]
        )
        self.lidar_filenames = sorted(
            [os.path.join(lidar_dir, filename) for filename in os.listdir(lidar_dir)]
        )
        print(len(self.img_filenames))
        print(len(self.lidar_filenames))
        # assert(len(self.img_filenames) == len(self.lidar_filenames))
        self.num_samples = len(self.img_filenames)

    def __len__(self):
        return self.num_samples

    def get_image(self, idx):
        assert idx < self.num_samples
        img_filename = self.img_filenames[idx]
        return utils.load_image(img_filename)

    def get_lidar(self, idx):
        assert idx < self.num_samples
        lidar_filename = self.lidar_filenames[idx]
        return utils.load_velo_scan(lidar_filename)

    def get_calibration(self, unused):
        return self.calib


def viz_kitti_video():
    video_path = os.path.join(ROOT_DIR, "dataset/2011_09_26/")
    dataset = kitti_object_video(
        os.path.join(video_path, "2011_09_26_drive_0023_sync/image_02/data"),
        os.path.join(video_path, "2011_09_26_drive_0023_sync/velodyne_points/data"),
        video_path,
    )
    print(len(dataset))
    for _ in range(len(dataset)):
        img = dataset.get_image(0)
        pc = dataset.get_lidar(0)
        cv2.imshow("video", img)
        draw_lidar(pc)
        raw_input()
        pc[:, 0:3] = dataset.get_calibration().project_velo_to_rect(pc[:, 0:3])
        draw_lidar(pc)
        raw_input()
    return


def show_image_with_boxes(img, objects, calib, show3d=True, depth=None):
    """ Show image with 2D bounding boxes """
    img1 = np.copy(img)  # for 2d bbox
    img2 = np.copy(img)  # for 3d bbox
    #img3 = np.copy(img)  # for 3d bbox
    #TODO: change the color of boxes
    for obj in objects:
        if obj.type == "DontCare":
            continue
        if obj.type == "Car":
            cv2.rectangle(
            img1,
            (int(obj.xmin), int(obj.ymin)),
            (int(obj.xmax), int(obj.ymax)),
            (0, 255, 0),
            2,
        )
        if obj.type == "Pedestrian":
            cv2.rectangle(
            img1,
            (int(obj.xmin), int(obj.ymin)),
            (int(obj.xmax), int(obj.ymax)),
            (255, 255, 0),
            2,
        )
        if obj.type == "Cyclist":
            cv2.rectangle(
            img1,
            (int(obj.xmin), int(obj.ymin)),
            (int(obj.xmax), int(obj.ymax)),
            (0, 255, 255),
            2,
        )
        box3d_pts_2d, _ = utils.compute_box_3d(obj, calib.P)
        if box3d_pts_2d is None:
            print("something wrong in the 3D box.")
            continue
        if obj.type == "Car":
            img2 = utils.draw_projected_box3d(img2, box3d_pts_2d, color=(0, 255, 0))
        elif obj.type == "Pedestrian":
            img2 = utils.draw_projected_box3d(img2, box3d_pts_2d, color=(255, 255, 0))
        elif obj.type == "Cyclist":
            img2 = utils.draw_projected_box3d(img2, box3d_pts_2d, color=(0, 255, 255))


        # project
        # box3d_pts_3d_velo = calib.project_rect_to_velo(box3d_pts_3d)
        # box3d_pts_32d = utils.box3d_to_rgb_box00(box3d_pts_3d_velo)
        # box3d_pts_32d = calib.project_velo_to_image(box3d_pts_3d_velo)
        # img3 = utils.draw_projected_box3d(img3, box3d_pts_32d)
    # print("img1:", img1.shape)
    cv2.imshow("2dbox", img1)
    # print("img3:",img3.shape)
    # Image.fromarray(img3).show()
    show3d = True
    if show3d:
        # print("img2:",img2.shape)
        cv2.imshow("3dbox", img2)
    if depth is not None:
        cv2.imshow("depth", depth)
    
    return img1, img2


def show_image_with_boxes_3type(img, objects, calib, objects2d, name, objects_pred):
    """ Show image with 2D bounding boxes """
    img1 = np.copy(img)  # for 2d bbox
    type_list = ["Pedestrian", "Car", "Cyclist"]
    # draw Label
    color = (0, 255, 0)
    for obj in objects:
        if obj.type not in type_list:
            continue
        cv2.rectangle(
            img1,
            (int(obj.xmin), int(obj.ymin)),
            (int(obj.xmax), int(obj.ymax)),
            color,
            3,
        )
    startx = 5
    font = cv2.FONT_HERSHEY_SIMPLEX

    text_lables = [obj.type for obj in objects if obj.type in type_list]
    text_lables.insert(0, "Label:")
    for n in range(len(text_lables)):
        text_pos = (startx, 25 * (n + 1))
        cv2.putText(img1, text_lables[n], text_pos, font, 0.5, color, 0, cv2.LINE_AA)
    # draw 2D Pred
    color = (0, 0, 255)
    for obj in objects2d:
        cv2.rectangle(
            img1,
            (int(obj.box2d[0]), int(obj.box2d[1])),
            (int(obj.box2d[2]), int(obj.box2d[3])),
            color,
            2,
        )
    startx = 85
    font = cv2.FONT_HERSHEY_SIMPLEX

    text_lables = [type_list[obj.typeid - 1] for obj in objects2d]
    text_lables.insert(0, "2D Pred:")
    for n in range(len(text_lables)):
        text_pos = (startx, 25 * (n + 1))
        cv2.putText(img1, text_lables[n], text_pos, font, 0.5, color, 0, cv2.LINE_AA)
    # draw 3D Pred
    if objects_pred is not None:
        color = (255, 0, 0)
        for obj in objects_pred:
            if obj.type not in type_list:
                continue
            cv2.rectangle(
                img1,
                (int(obj.xmin), int(obj.ymin)),
                (int(obj.xmax), int(obj.ymax)),
                color,
                1,
            )
        startx = 165
        font = cv2.FONT_HERSHEY_SIMPLEX

        text_lables = [obj.type for obj in objects_pred if obj.type in type_list]
        text_lables.insert(0, "3D Pred:")
        for n in range(len(text_lables)):
            text_pos = (startx, 25 * (n + 1))
            cv2.putText(
                img1, text_lables[n], text_pos, font, 0.5, color, 0, cv2.LINE_AA
            )

    cv2.imshow("with_bbox", img1)
    cv2.imwrite("imgs/" + str(name) + ".png", img1)


def get_lidar_in_image_fov(
    pc_velo, calib, xmin, ymin, xmax, ymax, return_more=False, clip_distance=2.0
):
    """ Filter lidar points, keep those in image FOV """
    pts_2d = calib.project_velo_to_image(pc_velo)
    fov_inds = (
        (pts_2d[:, 0] < xmax)
        & (pts_2d[:, 0] >= xmin)
        & (pts_2d[:, 1] < ymax)
        & (pts_2d[:, 1] >= ymin)
    )
    fov_inds = fov_inds & (pc_velo[:, 0] > clip_distance)
    imgfov_pc_velo = pc_velo[fov_inds, :]
    if return_more:
        return imgfov_pc_velo, pts_2d, fov_inds
    else:
        return imgfov_pc_velo


def get_lidar_index_in_image_fov(
    pc_velo, calib, xmin, ymin, xmax, ymax, return_more=False, clip_distance=2.0
):
    """ Filter lidar points, keep those in image FOV """
    pts_2d = calib.project_velo_to_image(pc_velo)
    fov_inds = (
        (pts_2d[:, 0] < xmax)
        & (pts_2d[:, 0] >= xmin)
        & (pts_2d[:, 1] < ymax)
        & (pts_2d[:, 1] >= ymin)
    )
    fov_inds = fov_inds & (pc_velo[:, 0] > clip_distance)
    return fov_inds


def depth_region_pt3d(depth, obj):
    b = obj.box2d
    # depth_region = depth[b[0]:b[2],b[2]:b[3],0]
    pt3d = []
    # import pdb; pdb.set_trace()
    for i in range(int(b[0]), int(b[2])):
        for j in range(int(b[1]), int(b[3])):
            pt3d.append([j, i, depth[j, i]])
    return np.array(pt3d)


def get_depth_pt3d(depth):
    pt3d = []
    for i in range(depth.shape[0]):
        for j in range(depth.shape[1]):
            pt3d.append([i, j, depth[i, j]])
    return np.array(pt3d)


def show_lidar_with_depth(
    pc_velo,
    objects,
    calib,
    fig,
    img_fov=False,
    img_width=None,
    img_height=None,
    objects_pred=None,
    depth=None,
    cam_img=None,
    constraint_box=False,
    pc_label=False,
    save=False,
):
    """ Show all LiDAR points.
        Draw 3d box in LiDAR point cloud (in velo coord system) """
    if "mlab" not in sys.modules:
        import mayavi.mlab as mlab
    from viz_util import draw_lidar_simple, draw_lidar, draw_gt_boxes3d

    print(("All point num: ", pc_velo.shape[0]))
    if img_fov:
        pc_velo_index = get_lidar_index_in_image_fov(
            pc_velo[:, :3], calib, 0, 0, img_width, img_height
        )
        pc_velo = pc_velo[pc_velo_index, :]
        print(("FOV point num: ", pc_velo.shape))
    print("pc_velo", pc_velo.shape)
    draw_lidar(pc_velo, fig=fig, pc_label=pc_label)

    # Draw depth
    if depth is not None:
        depth_pc_velo = calib.project_depth_to_velo(depth, constraint_box)

        indensity = np.ones((depth_pc_velo.shape[0], 1)) * 0.5
        depth_pc_velo = np.hstack((depth_pc_velo, indensity))
        print("depth_pc_velo:", depth_pc_velo.shape)
        print("depth_pc_velo:", type(depth_pc_velo))
        print(depth_pc_velo[:5])
        draw_lidar(depth_pc_velo, fig=fig, pts_color=(1, 1, 1))

        if save:
            data_idx = 0
            vely_dir = "data/object/training/depth_pc"
            save_filename = os.path.join(vely_dir, "%06d.bin" % (data_idx))
            print(save_filename)
            # np.save(save_filename+".npy", np.array(depth_pc_velo))
            depth_pc_velo = depth_pc_velo.astype(np.float32)
            depth_pc_velo.tofile(save_filename)

    # color = (0, 1, 0)
    for obj in objects:
        if obj.type == "DontCare":
            continue
        # Draw 3d bounding box
        _, box3d_pts_3d = utils.compute_box_3d(obj, calib.P)
        box3d_pts_3d_velo = calib.project_rect_to_velo(box3d_pts_3d)
        print("box3d_pts_3d_velo:")
        print(box3d_pts_3d_velo)

        #TODO: change the color of boxes
        if obj.type == "Car":
            draw_gt_boxes3d([box3d_pts_3d_velo], fig=fig, color=(0,1,0), label=obj.type)
        elif obj.type == "Pedestrian":
            draw_gt_boxes3d([box3d_pts_3d_velo], fig=fig, color=(0,1,1), label=obj.type)
        elif obj.type == "Cyclist":
            draw_gt_boxes3d([box3d_pts_3d_velo], fig=fig, color=(1,1,0), label=obj.type)


    if objects_pred is not None:
        color = (1, 0, 0)
        for obj in objects_pred:
            if obj.type == "DontCare":
                continue
            # Draw 3d bounding box
            _, box3d_pts_3d = utils.compute_box_3d(obj, calib.P)
            box3d_pts_3d_velo = calib.project_rect_to_velo(box3d_pts_3d)
            print("box3d_pts_3d_velo:")
            print(box3d_pts_3d_velo)
            draw_gt_boxes3d([box3d_pts_3d_velo], fig=fig, color=color)
            # Draw heading arrow
            _, ori3d_pts_3d = utils.compute_orientation_3d(obj, calib.P)
            ori3d_pts_3d_velo = calib.project_rect_to_velo(ori3d_pts_3d)
            x1, y1, z1 = ori3d_pts_3d_velo[0, :]
            x2, y2, z2 = ori3d_pts_3d_velo[1, :]
            mlab.plot3d(
                [x1, x2],
                [y1, y2],
                [z1, z2],
                color=color,
                tube_radius=None,
                line_width=1,
                figure=fig,
            )
    mlab.show(1)


def save_depth0(
    data_idx,
    pc_velo,
    calib,
    img_fov,
    img_width,
    img_height,
    depth,
    constraint_box=False,
):

    if img_fov:
        pc_velo_index = get_lidar_index_in_image_fov(
            pc_velo[:, :3], calib, 0, 0, img_width, img_height
        )
        pc_velo = pc_velo[pc_velo_index, :]
        type = np.zeros((pc_velo.shape[0], 1))
        pc_velo = np.hstack((pc_velo, type))
        print(("FOV point num: ", pc_velo.shape))
    # Draw depth
    if depth is not None:
        depth_pc_velo = calib.project_depth_to_velo(depth, constraint_box)

        indensity = np.ones((depth_pc_velo.shape[0], 1)) * 0.5
        depth_pc_velo = np.hstack((depth_pc_velo, indensity))

        type = np.ones((depth_pc_velo.shape[0], 1))
        depth_pc_velo = np.hstack((depth_pc_velo, type))
        print("depth_pc_velo:", depth_pc_velo.shape)

        depth_pc = np.concatenate((pc_velo, depth_pc_velo), axis=0)
        print("depth_pc:", depth_pc.shape)

    vely_dir = "data/object/training/depth_pc"
    save_filename = os.path.join(vely_dir, "%06d.bin" % (data_idx))

    depth_pc = depth_pc.astype(np.float32)
    depth_pc.tofile(save_filename)


def save_depth(
    data_idx,
    pc_velo,
    calib,
    img_fov,
    img_width,
    img_height,
    depth,
    constraint_box=False,
):

    if depth is not None:
        depth_pc_velo = calib.project_depth_to_velo(depth, constraint_box)

        indensity = np.ones((depth_pc_velo.shape[0], 1)) * 0.5
        depth_pc = np.hstack((depth_pc_velo, indensity))

        print("depth_pc:", depth_pc.shape)

    vely_dir = "data/object/training/depth_pc"
    save_filename = os.path.join(vely_dir, "%06d.bin" % (data_idx))

    depth_pc = depth_pc.astype(np.float32)
    depth_pc.tofile(save_filename)


def show_lidar_with_boxes(
    pc_velo,
    objects,
    calib,
    img_fov=False,
    img_width=None,
    img_height=None,
    objects_pred=None,
    depth=None,
    cam_img=None,
):
    """ Show all LiDAR points.
        Draw 3d box in LiDAR point cloud (in velo coord system) """
    if "mlab" not in sys.modules:
        import mayavi.mlab as mlab
    from viz_util import draw_lidar_simple, draw_lidar, draw_gt_boxes3d

    print(("All point num: ", pc_velo.shape[0]))
    fig = mlab.figure(
        figure=None, bgcolor=(0, 0, 0), fgcolor=None, engine=None, size=(1000, 500)
    )
    if img_fov:
        pc_velo = get_lidar_in_image_fov(
            pc_velo[:, 0:3], calib, 0, 0, img_width, img_height
        )
        print(("FOV point num: ", pc_velo.shape[0]))
    print("pc_velo", pc_velo.shape)
    draw_lidar(pc_velo, fig=fig)
    # pc_velo=pc_velo[:,0:3]

    color = (0, 1, 0)
    for obj in objects:
        if obj.type == "DontCare":
            continue
        # Draw 3d bounding box
        _, box3d_pts_3d = utils.compute_box_3d(obj, calib.P)
        box3d_pts_3d_velo = calib.project_rect_to_velo(box3d_pts_3d)
        print("box3d_pts_3d_velo:")
        print(box3d_pts_3d_velo)

        draw_gt_boxes3d([box3d_pts_3d_velo], fig=fig, color=color)

        # Draw depth
        if depth is not None:
            # import pdb; pdb.set_trace()
            depth_pt3d = depth_region_pt3d(depth, obj)
            depth_UVDepth = np.zeros_like(depth_pt3d)
            depth_UVDepth[:, 0] = depth_pt3d[:, 1]
            depth_UVDepth[:, 1] = depth_pt3d[:, 0]
            depth_UVDepth[:, 2] = depth_pt3d[:, 2]
            print("depth_pt3d:", depth_UVDepth)
            dep_pc_velo = calib.project_image_to_velo(depth_UVDepth)
            print("dep_pc_velo:", dep_pc_velo)

            draw_lidar(dep_pc_velo, fig=fig, pts_color=(1, 1, 1))

        # Draw heading arrow
        _, ori3d_pts_3d = utils.compute_orientation_3d(obj, calib.P)
        ori3d_pts_3d_velo = calib.project_rect_to_velo(ori3d_pts_3d)
        x1, y1, z1 = ori3d_pts_3d_velo[0, :]
        x2, y2, z2 = ori3d_pts_3d_velo[1, :]
        mlab.plot3d(
            [x1, x2],
            [y1, y2],
            [z1, z2],
            color=color,
            tube_radius=None,
            line_width=1,
            figure=fig,
        )
    if objects_pred is not None:
        color = (1, 0, 0)
        for obj in objects_pred:
            if obj.type == "DontCare":
                continue
            # Draw 3d bounding box
            _, box3d_pts_3d = utils.compute_box_3d(obj, calib.P)
            box3d_pts_3d_velo = calib.project_rect_to_velo(box3d_pts_3d)
            print("box3d_pts_3d_velo:")
            print(box3d_pts_3d_velo)
            draw_gt_boxes3d([box3d_pts_3d_velo], fig=fig, color=color)
            # Draw heading arrow
            _, ori3d_pts_3d = utils.compute_orientation_3d(obj, calib.P)
            ori3d_pts_3d_velo = calib.project_rect_to_velo(ori3d_pts_3d)
            x1, y1, z1 = ori3d_pts_3d_velo[0, :]
            x2, y2, z2 = ori3d_pts_3d_velo[1, :]
            mlab.plot3d(
                [x1, x2],
                [y1, y2],
                [z1, z2],
                color=color,
                tube_radius=None,
                line_width=1,
                figure=fig,
            )
    mlab.show(1)


def box_min_max(box3d):
    box_min = np.min(box3d, axis=0)
    box_max = np.max(box3d, axis=0)
    return box_min, box_max


def get_velo_whl(box3d, pc):
    bmin, bmax = box_min_max(box3d)
    ind = np.where(
        (pc[:, 0] >= bmin[0])
        & (pc[:, 0] <= bmax[0])
        & (pc[:, 1] >= bmin[1])
        & (pc[:, 1] <= bmax[1])
        & (pc[:, 2] >= bmin[2])
        & (pc[:, 2] <= bmax[2])
    )[0]
    # print(pc[ind,:])
    if len(ind) > 0:
        vmin, vmax = box_min_max(pc[ind, :])
        return vmax - vmin
    else:
        return 0, 0, 0, 0


def stat_lidar_with_boxes(pc_velo, objects, calib):
    """ Show all LiDAR points.
        Draw 3d box in LiDAR point cloud (in velo coord system) """

    # print(('All point num: ', pc_velo.shape[0]))

    # draw_lidar(pc_velo, fig=fig)
    # color=(0,1,0)
    for obj in objects:
        if obj.type == "DontCare":
            continue
        _, box3d_pts_3d = utils.compute_box_3d(obj, calib.P)
        box3d_pts_3d_velo = calib.project_rect_to_velo(box3d_pts_3d)
        v_l, v_w, v_h, _ = get_velo_whl(box3d_pts_3d_velo, pc_velo)
        print("%.4f %.4f %.4f %s" % (v_w, v_h, v_l, obj.type))


def show_lidar_on_image(pc_velo, img, calib, img_width, img_height):
    """ Project LiDAR points to image """
    img =  np.copy(img)
    imgfov_pc_velo, pts_2d, fov_inds = get_lidar_in_image_fov(
        pc_velo, calib, 0, 0, img_width, img_height, True
    )
    imgfov_pts_2d = pts_2d[fov_inds, :]
    imgfov_pc_rect = calib.project_velo_to_rect(imgfov_pc_velo)

    import matplotlib.pyplot as plt

    cmap = plt.cm.get_cmap("hsv", 256)
    cmap = np.array([cmap(i) for i in range(256)])[:, :3] * 255

    for i in range(imgfov_pts_2d.shape[0]):
        depth = imgfov_pc_rect[i, 2]
        color = cmap[int(640.0 / depth), :]
        cv2.circle(
            img,
            (int(np.round(imgfov_pts_2d[i, 0])), int(np.round(imgfov_pts_2d[i, 1]))),
            2,
            color=tuple(color),
            thickness=-1,
        )
    cv2.imshow("projection", img)
    return img


def show_lidar_topview_with_boxes(pc_velo, objects, calib, objects_pred=None):
    """ top_view image"""
    # print('pc_velo shape: ',pc_velo.shape)
    top_view = utils.lidar_to_top(pc_velo)
    top_image = utils.draw_top_image(top_view)
    print("top_image:", top_image.shape)
    # gt

    def bbox3d(obj):
        _, box3d_pts_3d = utils.compute_box_3d(obj, calib.P)
        box3d_pts_3d_velo = calib.project_rect_to_velo(box3d_pts_3d)
        return box3d_pts_3d_velo

    boxes3d = [bbox3d(obj) for obj in objects if obj.type != "DontCare"]
    gt = np.array(boxes3d)
    # print("box2d BV:",boxes3d)
    lines = [obj.type for obj in objects if obj.type != "DontCare"]
    top_image = utils.draw_box3d_on_top(
        top_image, gt, text_lables=lines, scores=None, thickness=1, is_gt=True
    )
    # pred
    if objects_pred is not None:
        boxes3d = [bbox3d(obj) for obj in objects_pred if obj.type != "DontCare"]
        gt = np.array(boxes3d)
        lines = [obj.type for obj in objects_pred if obj.type != "DontCare"]
        top_image = utils.draw_box3d_on_top(
            top_image, gt, text_lables=lines, scores=None, thickness=1, is_gt=False
        )

    cv2.imshow("top_image", top_image)
    return top_image


def dataset_viz(root_dir, args):
    dataset = kitti_object(root_dir, split=args.split, args=args)
    ## load 2d detection results
    #objects2ds = read_det_file("box2d.list")

    if args.show_lidar_with_depth:
        import mayavi.mlab as mlab

        fig = mlab.figure(
            figure=None, bgcolor=(0, 0, 0), fgcolor=None, engine=None, size=(1000, 500)
        )
    for data_idx in range(len(dataset)):
        if args.ind > 0:
            data_idx = args.ind
        # Load data from dataset
        if args.split == "training":
            objects = dataset.get_label_objects(data_idx)
        else:
            objects = []
        #objects2d = objects2ds[data_idx]

        objects_pred = None
        if args.pred:
            # if not dataset.isexist_pred_objects(data_idx):
            #    continue
            objects_pred = dataset.get_pred_objects(data_idx)
            if objects_pred == None:
                continue
        if objects_pred == None:
            print("no pred file")
            # objects_pred[0].print_object()

        n_vec = 4
        if args.pc_label:
            n_vec = 5

        dtype = np.float32
        if args.dtype64:
            dtype = np.float64
        pc_velo = dataset.get_lidar(data_idx, dtype, n_vec)[:, 0:n_vec]
        calib = dataset.get_calibration(data_idx)
        img = dataset.get_image(data_idx)
        img_height, img_width, _ = img.shape
        print(data_idx, "image shape: ", img.shape)
        print(data_idx, "velo  shape: ", pc_velo.shape)
        if args.depth:
            depth, _ = dataset.get_depth(data_idx)
            print(data_idx, "depth shape: ", depth.shape)
        else:
            depth = None

        # depth = cv2.cvtColor(depth, cv2.COLOR_BGR2RGB)
        # depth_height, depth_width, depth_channel = img.shape

        # print(('Image shape: ', img.shape))

        if args.stat:
            stat_lidar_with_boxes(pc_velo, objects, calib)
            continue
        print("======== Objects in Ground Truth ========")
        n_obj = 0
        for obj in objects:
            if obj.type != "DontCare":
                print("=== {} object ===".format(n_obj + 1))
                obj.print_object()
                n_obj += 1

        # Draw 3d box in LiDAR point cloud
        if args.show_lidar_topview_with_boxes:
            # Draw lidar top view
            show_lidar_topview_with_boxes(pc_velo, objects, calib, objects_pred)

        # show_image_with_boxes_3type(img, objects, calib, objects2d, data_idx, objects_pred)
        if args.show_image_with_boxes:
            # Draw 2d and 3d boxes on image
            show_image_with_boxes(img, objects, calib, True, depth)
        if args.show_lidar_with_depth:
            # Draw 3d box in LiDAR point cloud
            show_lidar_with_depth(
                pc_velo,
                objects,
                calib,
                fig,
                args.img_fov,
                img_width,
                img_height,
                objects_pred,
                depth,
                img,
                constraint_box=args.const_box,
                save=args.save_depth,
                pc_label=args.pc_label,
            )
            # show_lidar_with_boxes(pc_velo, objects, calib, True, img_width, img_height, \
            #    objects_pred, depth, img)
        if args.show_lidar_on_image:
            # Show LiDAR points on image.
            show_lidar_on_image(pc_velo[:, 0:3], img, calib, img_width, img_height)
        input_str = input()

        mlab.clf()
        if input_str == "killall":
            break





#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022/8/31 20:14:34

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import sys, types
import inspect
from pprint import pprint
# ===============================================================================
r'''提供一些调试方便的函数
'''
# ===============================================================================
__all__ = ['pprint', 'print_args','print_eval',
           'print_type', 'print_repr', 'print_signature',
           'dump_module_api',
           'dump_module_all',
           ]

def print_args(*args, **kws):
    print('args:', args)
    print('kws:', kws)

def print_eval(st):
    '''字符串打印并执行'''
    _frm = sys._getframe(1)
    val = eval(st, _frm.f_globals, _frm.f_locals)
    print( f"{st} = {val}")

def print_type(t):
    '''打印类别'''
    _type_name = None
    for tname in dir(types):
        if tname[0].isupper() and isinstance(t, getattr(types, tname)):
            _type_name = tname
            break
    if _type_name is None:
        _type_name = f"{type(t)}"
    print(_type_name)

def print_repr(v):
    '''打印 repr '''
    print(repr(v))

def print_signature(func):
    sig = inspect.signature(func)
    print(f'{func.__name__}{str(sig)}')

def dump_module_api(module, preffix=None, suffix=None, new_preffix=None, new_suffix=None):
    module_name = module.__name__.rpartition('.')[-1]

    lines = []
    apis = []
    for name in dir(module):
        if preffix and new_preffix and name.startswith(preffix):
            line = f"{new_preffix}{name[len(preffix):]} = {module_name}.{name} "
            lines.append(line)
            apis.append(f"{new_preffix}{name[len(preffix):]}")

        if suffix and new_suffix and name.endswith(suffix):
            line = f"{new_suffix}{name[:-len(suffix)]} = {module_name}.{name} "
            lines.append(line)
            apis.append(f"{new_suffix}{name[:-len(suffix)]}")

    print('__all__ = ', end='')
    pprint(apis)
    print('\n\n')

    for line in lines:
        print(line)

def dump_module_all(module=None, preffix=None, suffix=None):
    apis = []
    if module is None:
        module = sys.modules['__main__']
    for name in dir(module):
        if preffix and name.startswith(preffix):
            apis.append(name)

        if suffix and name.endswith(suffix):
            apis.append(name)

    print('__all__ = [')
    for name in apis:
        print(f"'{name}',")
    print(']')


#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022/9/17 23:37:17

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import os
import json
import numpy as np
from torch.utils.data import Dataset
import laok.util.log as klog
import laok.util.fs as kfs
import laok.util.parallel as kparal
import laok.cv3d as kcv3
# ===============================================================================
r'''
'''
# ===============================================================================

class ModelnetIneratia(Dataset):
    def __init__(self, root, split='train',
                 num_point = 1024,
                 num_category = 40,
                 transforms=None,
                 cached=False,
                 show_info = False,
                 ):
        self.root = root
        self.npoints = num_point
        self.num_category = num_category
        self.transforms = transforms
        self.split = split
        klog.debug(f'===== {self.__class__.__name__}')
        klog.debug(f'root={root}')
        klog.debug(f'split={split}, num_point={num_point}, num_category={num_category}')

        if self.num_category == 10:
            self.catfile = os.path.join(self.root, 'modelnet10_shape_names.txt')
        else:
            self.catfile = os.path.join(self.root, 'modelnet40_shape_names.txt')

        klog.debug(f'catfile={self.catfile}')

        self.cat = [line.rstrip() for line in open(self.catfile)]
        self.classes = dict(zip(self.cat, range(len(self.cat))))

        klog.debug(f'cat={self.cat}')
        klog.debug(f'classes={self.classes}')

        shape_ids = {}
        if self.num_category == 10:
            shape_ids['train'] = [line.rstrip() for line in open(os.path.join(self.root, 'modelnet10_train.txt'))]
            shape_ids['test'] = [line.rstrip() for line in open(os.path.join(self.root, 'modelnet10_test.txt'))]
        else:
            shape_ids['train'] = [line.rstrip() for line in open(os.path.join(self.root, 'modelnet40_train.txt'))]
            shape_ids['test'] = [line.rstrip() for line in open(os.path.join(self.root, 'modelnet40_test.txt'))]

        assert (split == 'train' or split == 'test')
        shape_names = ['_'.join(x.split('_')[0:-1]) for x in shape_ids[split]]
        self.datapath = [(shape_names[i], os.path.join(self.root, shape_names[i], shape_ids[split][i]) + '.txt') for i
                         in range(len(shape_ids[split]))]
        print('The size of %s data is %d' % (split, len(self.datapath)))
        self.show_info = show_info

    def __len__(self):
        return len(self.datapath)

    def _get_item(self, index):
        name, ptfile = self.datapath[index]

        if self.show_info:
            klog.info(f'{name} point-file:{ptfile}')

        cls = self.classes[name]
        label = np.array([cls]).astype(np.int32)

        point_set = np.loadtxt(ptfile, delimiter=',').astype(np.float32)
        # point_set = kcv3.farthest_point_sample(point_set, self.npoints)
        if self.split == 'train':
            point_set = kcv3.random_sample(point_set, self.npoints)
        else:
            point_set = point_set[:self.npoints, :]

        point_set[:, 0:3] = kcv3.pc_normalize(point_set[:, 0:3])

        interia_file = kfs.path_replace_ext(ptfile, 'inertia')
        if self.show_info:
            klog.info(f'interia_file: {interia_file}')

        with open(interia_file) as f:
            jdata = json.load(f)
            eccentricity = np.array(jdata['Eccentricity'], dtype=np.float32)
            interia = np.array(jdata['MomentOfInertia'], dtype=np.float32)

        if self.transforms:
            point_set = self.transforms(point_set)

        return point_set, label[0], interia, eccentricity

    def __getitem__(self, index):
        '''
        :param index:
        :return:  point_set, label, interia, eccentricity
        '''
        return self._get_item(index)


if __name__ == '__main__':
    import torch
    import laok.util.torch_ as ktorch
    import laok.util.conf as kconf

    data = ModelnetIneratia(kconf.get_conf('datasets').ModelnetIneratia,  num_point=1024, show_info=True, cached=True)
    # DataLoader = torch.utils.data.DataLoader(data, batch_size=2, shuffle=False)
    # for point, label, interia, eccentricity in DataLoader:
    #     ktorch.pr(point)
    #     ktorch.pr(label)
    #     ktorch.pr(interia)
    #     ktorch.pr(eccentricity)
    #     kcv3.show_cld_xyz(point[0])
    #     break



#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2021/3/17 10:36:20

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import random
from ..fs import file_read_lines, path_replace_basename
#===============================================================================
#
#===============================================================================
__all__ = ['color_rand', 'color_rand_seed']

_color_map = {}
_color_set = set()
_rand = random.Random()

def _add_color(color):
    _color_map[len(_color_map)] = color
    _color_set.add(color)

def _init_color_list():
    if not _color_map:
        fname = path_replace_basename(__file__, 'color_list.txt')
        for line in file_read_lines(fname):
            ds = line.split(',')
            r,g,b = int(ds[0]), int(ds[1]), int(ds[2])
            _add_color((r,g,b))

def _gen_color():
    for i in range(3):
        r = _rand.randint(0,255)
        g = _rand.randint(0,255)
        b = _rand.randint(0,255)
        c = (r,g,b)
        if c in _color_set:
            continue
        return c

def color_rand(idx = None):
    '''
    :param idx:  color index, when None, rand a color
    :return:
    '''
    if idx is None:
        return _gen_color()
    else:
        idx = int(idx)
        _init_color_list()
        if idx in _color_map:
            return _color_map[idx]
        c = _gen_color()
        _add_color(c)
        return c

def color_rand_seed(seed):
    _rand.seed(seed)

if __name__ == "__main__":
    res = color_rand()
    print(res)
    res = color_rand()
    print(res)
    res = color_rand(10)
    print(res)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/7/12 21:47:37

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import cv2
# ===============================================================================
r'''
'''
# ===============================================================================
__all__ = ['morph_open', 'morph_close',
           'morph_erode', 'morph_dilate',
           'morph_gradient', 'morph_hitmiss',
           'morph_tophat', 'morph_blackhat',
           'kernel_rect', 'kernel_cross', 'kernel_ellipse',
]

def kernel_rect(ksize=(3, 3)):
    return cv2.getStructuringElement(shape=cv2.MORPH_RECT, ksize=ksize)

def kernel_cross(ksize=(3, 3)):
    return cv2.getStructuringElement(shape=cv2.MORPH_CROSS, ksize=ksize)

def kernel_ellipse(ksize=(3, 3)):
    return cv2.getStructuringElement(shape=cv2.MORPH_ELLIPSE, ksize=ksize)

def morph_open(img, kernel=None, iterations=1, borderType=cv2.BORDER_CONSTANT, borderValue=(0,0,0)):
    return cv2.morphologyEx(img, op=cv2.MORPH_OPEN, kernel=kernel, iterations=iterations, borderType=borderType, borderValue=borderValue)

def morph_close(img, kernel=None, iterations=1, borderType=cv2.BORDER_CONSTANT, borderValue=(0,0,0)):
    return cv2.morphologyEx(img, op=cv2.MORPH_CLOSE, kernel=kernel, iterations=iterations, borderType=borderType, borderValue=borderValue)

def morph_erode(img, kernel=None, iterations=1, borderType=cv2.BORDER_CONSTANT, borderValue=(0,0,0)):
    return cv2.morphologyEx(img, op=cv2.MORPH_ERODE, kernel=kernel, iterations=iterations, borderType=borderType, borderValue=borderValue)

def morph_dilate(img, kernel=None, iterations=1, borderType=cv2.BORDER_CONSTANT, borderValue=(0,0,0)):
    return cv2.morphologyEx(img, op=cv2.MORPH_DILATE, kernel=kernel, iterations=iterations, borderType=borderType, borderValue=borderValue)

def morph_gradient(img, kernel=None, iterations=1, borderType=cv2.BORDER_CONSTANT, borderValue=(0,0,0)):
    return cv2.morphologyEx(img, op=cv2.MORPH_GRADIENT, kernel=kernel, iterations=iterations, borderType=borderType, borderValue=borderValue)

def morph_tophat(img, kernel=None, iterations=1, borderType=cv2.BORDER_CONSTANT, borderValue=(0,0,0)):
    return cv2.morphologyEx(img, op=cv2.MORPH_TOPHAT, kernel=kernel, iterations=iterations, borderType=borderType, borderValue=borderValue)

def morph_blackhat(img, kernel=None, iterations=1, borderType=cv2.BORDER_CONSTANT, borderValue=(0,0,0)):
    return cv2.morphologyEx(img, op=cv2.MORPH_BLACKHAT, kernel=kernel, iterations=iterations, borderType=borderType, borderValue=borderValue)

def morph_hitmiss(img, kernel=None, iterations=1):
    return cv2.morphologyEx(img, op=cv2.MORPH_HITMISS, kernel=kernel, iterations=iterations)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/7/12 20:24:46

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import cv2
# ===============================================================================
r'''
'''
# ===============================================================================
__all__ = ['filter_2d',  'filter_median', 'filter_box', 'filter_gauss',
            'filter_bilateral'
]

def filter_2d(img, kernel, delta=0, borderType=cv2.BORDER_REPLICATE):
    return cv2.filter2D(img, ddepth=-1, kernel=kernel, delta=delta, borderType=borderType)

def filter_median(img, ksize=3):
    return cv2.medianBlur(img, ksize=ksize)

def filter_box(img, ksize=(3,3), borderType=cv2.BORDER_REPLICATE ):
    return cv2.boxFilter(img, ddepth=-1, ksize=ksize, normalize=True, borderType=borderType)

def filter_gauss(img, ksize=(3,3), sigmaX=0, sigmaY=0, borderType=cv2.BORDER_REPLICATE):
    return cv2.GaussianBlur(img, ksize=ksize, sigmaX=sigmaX, sigmaY=sigmaY, borderType=borderType)

def filter_bilateral(img, d=5, sigmaColor=30, sigmaSpace=30, borderType=cv2.BORDER_REPLICATE):
    return cv2.bilateralFilter(img, d=d, sigmaColor=sigmaColor, sigmaSpace=sigmaSpace, borderType=borderType)







#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/4/19 19:43:45
@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import requests
from laok.base.fs import file_write_bin, path_exist
from laok.base.net import url_make_file
#===============================================================================
r'''requests工具代码
'''
#===============================================================================
__all__ = ['random_user_agent', 'req', 'req_text', 'req_json', 'req_content', 'req_save', 'req_save_auto']

_random_user_agent = False
def random_user_agent():
    ''' 拦截请求的参数,将headers修改为 自定义的 user_agent
    :return:
    '''
    global _random_user_agent
    if _random_user_agent:
        return
    _random_user_agent = True
    user_agent_pc = [
        # 谷歌
        'Mozilla/5.0.html (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.html.2171.71 Safari/537.36',
        'Mozilla/5.0.html (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.html.1271.64 Safari/537.11',
        'Mozilla/5.0.html (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.16 (KHTML, like Gecko) Chrome/10.0.html.648.133 Safari/534.16',
        # 火狐
        'Mozilla/5.0.html (Windows NT 6.1; WOW64; rv:34.0.html) Gecko/20100101 Firefox/34.0.html',
        'Mozilla/5.0.html (X11; U; Linux x86_64; zh-CN; rv:1.9.2.10) Gecko/20100922 Ubuntu/10.10 (maverick) Firefox/3.6.10',
        # opera
        'Mozilla/5.0.html (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.html.2171.95 Safari/537.36 OPR/26.0.html.1656.60',
        # qq浏览器
        'Mozilla/5.0.html (compatible; MSIE 9.0.html; Windows NT 6.1; WOW64; Trident/5.0.html; SLCC2; .NET CLR 2.0.html.50727; .NET CLR 3.5.30729; .NET CLR 3.0.html.30729; Media Center PC 6.0.html; .NET4.0C; .NET4.0E; QQBrowser/7.0.html.3698.400)',
        # 搜狗浏览器
        'Mozilla/5.0.html (Windows NT 5.1) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.html.963.84 Safari/535.11 SE 2.X MetaSr 1.0.html',
        # 360浏览器
        'Mozilla/5.0.html (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.html.1599.101 Safari/537.36',
        'Mozilla/5.0.html (Windows NT 6.1; WOW64; Trident/7.0.html; rv:11.0.html) like Gecko',
        # uc浏览器
        'Mozilla/5.0.html (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/38.0.html.2125.122 UBrowser/4.0.html.3214.0.html Safari/537.36',
    ]
    import random
    _req = requests.Session.request
    def user_agent_req(*args, **kwargs):
        headers = kwargs.get('headers')
        if not headers:
            headers = {}
        headers['User-Agent'] = random.choice(user_agent_pc)
        kwargs['headers'] = headers
        return _req(*args, **kwargs)
    requests.Session.request = user_agent_req

def req(url, method='get', **kws):
    '''设置随机 user-agent,然后执行请求
    '''
    random_user_agent()
    return requests.request(method=method, url=url, **kws)

def req_text(url, method='get', **kws):
    '''返回 text '''
    return req(url=url, method=method, **kws).text

def req_json(url, method='get', **kws):
    '''返回 json '''
    return req(url=url, method=method, **kws).json()

def req_content(url, method='get', **kws):
    '''返回二进制内容'''
    return req(url=url, method=method, **kws).content

def req_save(fname, url, method='get', skip_exist=False, **kws):
    '''baocun数据'''
    if skip_exist and path_exist(fname):
        return fname
    data = req_content(url=url, method=method, **kws)
    file_write_bin(fname, data)
    return fname

def req_save_auto(url, method='get', skip_exist=False, **kws):
    fname = url_make_file(url)
    req_save(fname=fname, url=url, method=method, skip_exist=skip_exist, **kws)
    return fname




#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/7/11 15:34:31

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import cv2
from .info import is_gray, is_color, image_size
# ===============================================================================
r'''# Opencv  图像格式        通道：BGR 像素值：[0,255]        (h,w)
    gray是 (h,w)  
    color是(h,w,3)
cv2.INTER_LINEAR  = 1
'''
# ===============================================================================
__all__ = [ 'gray2bgr', 'gray2rgb',
            'bgr2gray', 'rgb2gray',
            'swap_rb',
            'keep_bgr', 'keep_gray',
            'resize_abs', 'resize_rel'
           ]

def gray2bgr(img):
    if img.ndim == 2:
        img = cv2.cvtColor(img, cv2.COLOR_GRAY2BGR)
    return img

def gray2rgb(img):
    if img.ndim == 2:
        img = cv2.cvtColor(img, cv2.COLOR_GRAY2RGB)
    return img

def bgr2gray(img):
    if img.ndim == 3:
        if img.shape[-1] == 3:
            img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
        elif img.shape[-1] == 4:
            img = cv2.cvtColor(img, cv2.COLOR_BGRA2GRAY)
    return img

def rgb2gray(img):
    if img.ndim == 3:
        if img.shape[-1] == 3:
            img = cv2.cvtColor(img, cv2.COLOR_RGB2GRAY)
        elif img.shape[-1] == 4:
            img = cv2.cvtColor(img, cv2.COLOR_RGBA2GRAY)
    return img

def swap_rb(img):
    if img.ndim == 3:
        img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
    return img

def norm_minmax_u8(mat, alpha=0, beta=255):
    return cv2.normalize(mat, dst=None, alpha=alpha, beta=beta, norm_type=cv2.NORM_MINMAX, dtype=cv2.CV_8U)

def norm_minmax_f32(mat, alpha=0, beta=1):
    return cv2.normalize(mat, dst=None, alpha=alpha, beta=beta, norm_type=cv2.NORM_MINMAX, dtype=cv2.CV_32F)

def norm_minmax_i32(mat, alpha=0, beta=255):
    return cv2.normalize(mat, dst=None, alpha=alpha, beta=beta, norm_type=cv2.NORM_MINMAX, dtype=cv2.CV_32S)

def keep_gray(img):
    if is_color(img):
        return bgr2gray(img)
    return img

def keep_bgr(img):
    if is_gray(img):
        return gray2bgr(img)
    return img

def resize_abs(img, width, height=0, interpolation=cv2.INTER_LINEAR):
    if height == 0:
        height = width
    return cv2.resize(img, dsize=(int(width), int(height)), interpolation=interpolation)

def resize_rel(img, fx=0.5, fy=0, interpolation=cv2.INTER_LINEAR):
    if fy == 0:
        fy = fx
    return cv2.resize(img, fx=fx, fy=fy, interpolation=interpolation)




#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022/10/18 02:35:56

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import torch
from laok.base.str import fmt_size
# ===============================================================================
r'''
'''
# ===============================================================================
__all__ = ['print_data']


def print_data(data, **kws):
    if isinstance(data, torch.TensorType):
        _print_tensor(data, **kws)
    elif isinstance(data, torch.Module):
        _print_model(data, **kws)

def _print_tensor(data, title=None, show_data=True):
    '''print detail data information
    '''

    if title is None:
        msg = f"===="
    else:
        msg = f"====[{title}] "

    if isinstance(data, torch.Tensor):
        _type = 'Tensor'
    else:
        _type = type(data)

    msg += f'type[{_type}]'
    if isinstance(data, torch.Tensor):
        msg += f' shape{fmt_size(data.shape)} dim[{data.ndim}] dtype[{_fmt_dtype(data.dtype)}] device[{data.device}]'

    msg += f' id[{id(data)}]\n'

    if show_data:
        msg += f'{data}'

    print(msg)
    return data

def _print_model(model):
    print(f'=== name: {model.__class__.__name__}')
    for i, (name, param) in enumerate(model.named_parameters()):
        print( f"<{i}> {name}")
        print( f"    requires_grad:{param.requires_grad}  size={fmt_size(param.size())}")
    return model

def _fmt_dtype(dt):
    return str(dt).replace('torch.', '')


#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/8/28 19:30:19

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import cv2
from .info import image_size
#===============================================================================
r'''
'''
#===============================================================================
__all__ = ['resize_rel',
           'resize_abs', 'resize_to_width', 'resize_to_height',
           ]

def resize_rel(img, rw=0, rh=0, interpolation=cv2.INTER_LINEAR):
    if rw == 0 and rh == 0:
        return img
    if rw == 0:
        rw = rh
    if rh == 0:
        rh = rw
    return cv2.resize(img, fx=rw, fy=rh, interpolation=interpolation)

def resize_abs(img, w=0, h=0, interpolation=cv2.INTER_LINEAR):
    w, h = int(w), int(h)
    w2, h2 = image_size(img)
    if w == 0 and h == 0:
        return img
    if w == w2 and h == h2:
        return img
    if w == 0:
        w = w2
    if h == 0:
        h = h2
    return cv2.resize(img, dsize=(w, h), interpolation=interpolation)

def resize_to_width(img, w, interpolation=cv2.INTER_LINEAR):
    w2, h2 = image_size(img)
    h = w / w2 * h2
    return resize_abs(img, w, h, interpolation)

def resize_to_height(img, h, interpolation=cv2.INTER_LINEAR):
    w2, h2 = image_size(img)
    w = h / h2 * w2
    return resize_abs(img, w, h, interpolation)




#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/4/19 16:20:56
@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import torch
#===============================================================================
r'''
'''
#===============================================================================
__all__ = ['get_device', 'set_device']

def get_device(obj=None):
    '''自动获取对象的设备类型
    :param obj: 对象
    :return:
    '''
    if obj is None:
        obj = "cuda" if torch.cuda.is_available() else "cpu"

    if isinstance(obj, str):
        return torch.device(obj)

    if isinstance(obj, torch.Tensor):
        return obj.device

    if isinstance(obj, torch.nn.Module):
        p = next(obj.parameters())
        return p.device

    if isinstance(obj, torch.device):
        return obj

def set_device(src, dst):
    device = get_device(dst)
    return src.to(device)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/8/6 18:29:31

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import matplotlib.pyplot as plt
# ===============================================================================
r'''
'''
# ===============================================================================

axes_title = plt.title

axes_legend = plt.legend

axes_xlabel = plt.xlabel
axes_ylable = plt.ylabel

axes_xlim = plt.xlim
axes_ylim = plt.ylim

axes_xscale = plt.xscale
axes_yscale = plt.yscale

axes_xticks = plt.xticks
axes_yticks = plt.yticks





from ._dropout import *
from ._jitter import *
from ._normal import *
from ._rotate import *
from ._sample import *
from ._scale import *
from ._swap_filed import *
from ._torch import *
from ._translate import *


#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/7/11 15:40:10

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''

# ===============================================================================
r'''
'''
# ===============================================================================
__all__ = ['is_color', 'is_gray',
           'image_size', 'image_width',
           'image_height', 'image_center',
           'image_channels']

def is_color(img):
    return img.ndim == 3 and img.shape[-1] > 1

def is_gray(img):
    return not is_color(img)

def image_size(img):
    '''width, height'''
    return (img.shape[1], img.shape[0])

def image_width(img):
    return img.shape[1]

def image_height(img):
    return img.shape[0]

def image_center(img):
    height, width = img.shape[0:2]
    return (width//2, height//2)

def image_channels(img):
    if img.ndim == 2:
        return 1
    if img.ndim == 3:
        return img.shape[-1]
    assert("error img format")




#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/8/2 22:28:29

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
from sklearn import linear_model
from sklearn import discriminant_analysis
from sklearn import svm
from sklearn import neighbors
from sklearn import gaussian_process
from sklearn import naive_bayes
from sklearn import tree
from sklearn import ensemble
from sklearn import neural_network
# ===============================================================================
r'''
'''
# ===============================================================================
__all__ = [
 'ClassifierAdaBoost',
 'ClassifierBagging',
 'ClassifierBernoulliNB',
 'ClassifierCategoricalNB',
 'ClassifierComplementNB',
 'ClassifierDecisionTree',
 'ClassifierExtraTree',
 'ClassifierExtraTrees',
 'ClassifierGaussianNB',
 'ClassifierGaussianProcess',
 'ClassifierGradientBoosting',
 'ClassifierHistGradientBoosting',
 'ClassifierIsolationForest',
 'ClassifierKNeighbors',
 'ClassifierLDA',
 'ClassifierLinearSVC',
 'ClassifierLogisticRegression',
 'ClassifierLogisticRegressionCV',
 'ClassifierMLP',
 'ClassifierMultinomialNB',
 'ClassifierNearestCentroid',
 'ClassifierNuSVC',
 'ClassifierOneClassSVM',
 'ClassifierPassiveAggressive',
 'ClassifierPerceptron',
 'ClassifierQDA',
 'ClassifierRadiusNeighbors',
 'ClassifierRandomForest',
 'ClassifierRidge',
 'ClassifierSGD',
 'ClassifierSGDOneClassSVM',
 'ClassifierSVC',
 'ClassifierStacking',
 'ClassifierVoting',
 ]

# linear_model
ClassifierRidge = linear_model.RidgeClassifier
ClassifierLogisticRegression = linear_model.LogisticRegression
ClassifierLogisticRegressionCV = linear_model.LogisticRegressionCV
ClassifierSGD = linear_model.SGDClassifier
ClassifierPerceptron = linear_model.Perceptron
ClassifierPassiveAggressive = linear_model.PassiveAggressiveClassifier
ClassifierSGDOneClassSVM = linear_model.SGDOneClassSVM


# discriminant_analysis
ClassifierLDA = discriminant_analysis.LinearDiscriminantAnalysis
ClassifierQDA = discriminant_analysis.QuadraticDiscriminantAnalysis

# svm
ClassifierSVC = svm.SVC
ClassifierNuSVC = svm.NuSVC
ClassifierLinearSVC = svm.LinearSVC
ClassifierOneClassSVM = svm.OneClassSVM

# neighbors
ClassifierKNeighbors = neighbors.KNeighborsClassifier
ClassifierRadiusNeighbors = neighbors.RadiusNeighborsClassifier
ClassifierNearestCentroid = neighbors.NearestCentroid

# gaussian_process
ClassifierGaussianProcess = gaussian_process.GaussianProcessClassifier

# naive_bayes
ClassifierBernoulliNB = naive_bayes.BernoulliNB
ClassifierGaussianNB = naive_bayes.GaussianNB
ClassifierMultinomialNB = naive_bayes.MultinomialNB
ClassifierComplementNB = naive_bayes.ComplementNB
ClassifierCategoricalNB = naive_bayes.CategoricalNB

# tree
ClassifierDecisionTree = tree.DecisionTreeClassifier
ClassifierExtraTree = tree.ExtraTreeClassifier

# ensemble
ClassifierRandomForest = ensemble.RandomForestClassifier
ClassifierExtraTrees = ensemble.ExtraTreesClassifier
ClassifierBagging = ensemble.BaggingClassifier
ClassifierIsolationForest = ensemble.IsolationForest
ClassifierGradientBoosting = ensemble.GradientBoostingClassifier
ClassifierAdaBoost = ensemble.AdaBoostClassifier
ClassifierVoting = ensemble.VotingClassifier
ClassifierStacking = ensemble.StackingClassifier
ClassifierHistGradientBoosting = ensemble.HistGradientBoostingClassifier

# neural_network
ClassifierMLP = neural_network.MLPClassifier




#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/7/12 16:58:45

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import math
import matplotlib.pyplot as plt
import numpy as np
# ===============================================================================
r'''
'''
# ===============================================================================
__all__ = ['show_image',
           'show_image_list',
           'show_regression',
           'show_clusters']

def show_image(img, label=None, img_kws=None, is_opencv=False, **plt_kws):
    '''显示图像
    '''
    fig, axes = plt.subplots(**plt_kws)

    if label:
        axes.set_title(label)

    if img_kws is None:
        img_kws = {}

    if is_opencv:
        if img.ndim == 2:
            img_kws.setdefault('cmap', 'gray')
        if img.ndim == 3:
            # im_swap = img.copy()
            # im_swap[:, :, 0], im_swap[:, :, 2] = img[:, :, 2], img[:, :, 0]
            img = img[:, :, ::-1]

    axes.imshow(img, **img_kws)

    axes.get_xaxis().set_visible(False) #隐藏x轴
    axes.get_yaxis().set_visible(False) #隐藏y轴

    plt.show()


def show_image_list(images, labels=None, ncols=1, img_kws=None, is_opencv=False, **plt_kws):
    '''显示多张图片
    '''
    nrows = int( math.ceil(len(images) / ncols) )
    fig, axes = plt.subplots(nrows=nrows, ncols=ncols, **plt_kws)

    axes.resize(nrows * ncols)
    if img_kws is None:
        img_kws = {}

    for i, f in enumerate(axes):
        if i < len(images):

            img = images[i]
            if is_opencv:
                if img.ndim == 2:
                    img_kws.setdefault('cmap', 'gray')
                if img.ndim == 3:
                    # img = img[:, :, [2,1,0]]
                    img = img[:, :, ::-1]

            f.imshow(img, **img_kws)

            if labels and i < len(labels):
                f.set_title(labels[i])
        else:
            f.spines['top'].set_visible(False)
            f.spines['right'].set_visible(False)
            f.spines['bottom'].set_visible(False)
            f.spines['left'].set_visible(False)

        f.get_xaxis().set_visible(False)
        f.get_yaxis().set_visible(False)

    plt.show()


def show_regression(X_test, y_test, y_pred, title=None):
    '''显示 数据点 和 回归线条,只支持显示1维数据
    '''
    plt.scatter(X_test, y_test, color="black")
    plt.plot(X_test, y_pred, color="blue", linewidth=3)
    if title:
        plt.title(title)
    plt.show()


def show_clusters(X, labels, indexes=(0, 1)):
    '''显示 类别
    '''
    clusters = np.unique(labels)
    x, y = indexes
    for cluster in clusters:
        row_ix = np.where(labels == cluster)
        plt.scatter(X[row_ix, x], X[row_ix, y])
    plt.show()


#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2023/8/2 23:04:07

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
from sklearn import metrics
# ===============================================================================
r'''评分函数
'''
# ===============================================================================
__all__ = ['score_accuracy',
 'score_adjusted_mutual_info',
 'score_adjusted_rand',
 'score_average_precision',
 'score_balanced_accuracy',
 'score_calinski_harabasz',
 'score_cohen_kappa',
 'score_completeness',
 'score_consensus',
 'score_d2_absolute_error',
 'score_d2_pinball',
 'score_d2_tweedie',
 'score_davies_bouldin',
 'score_dcg',
 'score_explained_variance',
 'score_f1',
 'score_fbeta',
 'score_fowlkes_mallows',
 'score_homogeneity',
 'score_jaccard',
 'score_label_ranking_average_precision',
 'score_mutual_info',
 'score_ndcg',
 'score_normalized_mutual_info',
 'score_precision',
 'score_r2',
 'score_rand',
 'score_recall',
 'score_roc_auc',
 'score_silhouette',
 'score_top_k_accuracy',
 'score_v_measure']



score_accuracy = metrics.accuracy_score
score_adjusted_mutual_info = metrics.adjusted_mutual_info_score
score_adjusted_rand = metrics.adjusted_rand_score
score_average_precision = metrics.average_precision_score
score_balanced_accuracy = metrics.balanced_accuracy_score
score_calinski_harabasz = metrics.calinski_harabasz_score
score_cohen_kappa = metrics.cohen_kappa_score
score_completeness = metrics.completeness_score
score_consensus = metrics.consensus_score
score_d2_absolute_error = metrics.d2_absolute_error_score
score_d2_pinball = metrics.d2_pinball_score
score_d2_tweedie = metrics.d2_tweedie_score
score_davies_bouldin = metrics.davies_bouldin_score
score_dcg = metrics.dcg_score
score_explained_variance = metrics.explained_variance_score
score_f1 = metrics.f1_score
score_fbeta = metrics.fbeta_score
score_fowlkes_mallows = metrics.fowlkes_mallows_score
score_homogeneity = metrics.homogeneity_score
score_jaccard = metrics.jaccard_score
score_label_ranking_average_precision = metrics.label_ranking_average_precision_score
score_mutual_info = metrics.mutual_info_score
score_ndcg = metrics.ndcg_score
score_normalized_mutual_info = metrics.normalized_mutual_info_score
score_precision = metrics.precision_score
score_r2 = metrics.r2_score
score_rand = metrics.rand_score
score_recall = metrics.recall_score
score_roc_auc = metrics.roc_auc_score
score_silhouette = metrics.silhouette_score
score_top_k_accuracy = metrics.top_k_accuracy_score
score_v_measure = metrics.v_measure_score






from setuptools import setup, find_packages

setup(
    name='laok',
    version='0.2.26',
    keywords=['_lk', 'dump', 'run'],
    license='MIT License',
    author='Liu Kuan',
    author_email='1306743659@qq.com',
    description='laok utils library, simplify my daily programming work',
    packages=find_packages(),
    include_package_data=True,
    package_data = {"":["*.*"]}
)
